##
## This file is part of the libsigrokdecode project.
##
## Copyright (C) 2017 Gerhard Sittig <gerhard.sittig@gmx.net>
##
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, see <http://www.gnu.org/licenses/>.
##

# This implementation is incomplete. TODO items:
# - Support the optional RESET# pin, detect cold and warm reset.
# - Split slot values into audio samples of their respective width and
#   frequency (either on user provided parameters, or from inspection of
#   decoded register access).

import sigrokdecode as srd

class ChannelError(Exception):
    pass

class Pins:
    (SYNC, BIT_CLK, SDATA_OUT, SDATA_IN, RESET) = range(5)

class Ann:
    (
        BITS_OUT, BITS_IN,
        SLOT_OUT_RAW, SLOT_OUT_TAG, SLOT_OUT_ADDR, SLOT_OUT_DATA,
        SLOT_OUT_03, SLOT_OUT_04, SLOT_OUT_05, SLOT_OUT_06,
        SLOT_OUT_07, SLOT_OUT_08, SLOT_OUT_09, SLOT_OUT_10,
        SLOT_OUT_11, SLOT_OUT_IO,
        SLOT_IN_RAW, SLOT_IN_TAG, SLOT_IN_ADDR, SLOT_IN_DATA,
        SLOT_IN_03, SLOT_IN_04, SLOT_IN_05, SLOT_IN_06,
        SLOT_IN_07, SLOT_IN_08, SLOT_IN_09, SLOT_IN_10,
        SLOT_IN_11, SLOT_IN_IO,
        WARN, ERROR,
    ) = range(32)
    (
        BIN_FRAME_OUT,
        BIN_FRAME_IN,
        BIN_SLOT_RAW_OUT,
        BIN_SLOT_RAW_IN,
    ) = range(4)

class Decoder(srd.Decoder):
    api_version = 3
    id = 'ac97'
    name = "AC '97"
    longname = "Audio Codec '97"
    desc = 'Audio and modem control for PC systems.'
    license = 'gplv2+'
    inputs = ['logic']
    outputs = []
    tags = ['Audio', 'PC']
    channels = (
        {'id': 'sync', 'name': 'SYNC', 'desc': 'Frame synchronization'},
        {'id': 'clk', 'name': 'BIT_CLK', 'desc': 'Data bits clock'},
    )
    optional_channels = (
        {'id': 'out', 'name': 'SDATA_OUT', 'desc': 'Data output'},
        {'id': 'in', 'name': 'SDATA_IN', 'desc': 'Data input'},
        {'id': 'rst', 'name': 'RESET#', 'desc': 'Reset line'},
    )
    annotations = (
        ('bit-out', 'Output bits'),
        ('bit-in', 'Input bits'),
        ('slot-out-raw', 'Output raw value'),
        ('slot-out-tag', 'Output TAG'),
        ('slot-out-cmd-addr', 'Output command address'),
        ('slot-out-cmd-data', 'Output command data'),
        ('slot-out-03', 'Output slot 3'),
        ('slot-out-04', 'Output slot 4'),
        ('slot-out-05', 'Output slot 5'),
        ('slot-out-06', 'Output slot 6'),
        ('slot-out-07', 'Output slot 7'),
        ('slot-out-08', 'Output slot 8'),
        ('slot-out-09', 'Output slot 9'),
        ('slot-out-10', 'Output slot 10'),
        ('slot-out-11', 'Output slot 11'),
        ('slot-out-io-ctrl', 'Output I/O control'),
        ('slot-in-raw', 'Input raw value'),
        ('slot-in-tag', 'Input TAG'),
        ('slot-in-sts-addr', 'Input status address'),
        ('slot-in-sts-data', 'Input status data'),
        ('slot-in-03', 'Input slot 3'),
        ('slot-in-04', 'Input slot 4'),
        ('slot-in-05', 'Input slot 5'),
        ('slot-in-06', 'Input slot 6'),
        ('slot-in-07', 'Input slot 7'),
        ('slot-in-08', 'Input slot 8'),
        ('slot-in-09', 'Input slot 9'),
        ('slot-in-10', 'Input slot 10'),
        ('slot-in-11', 'Input slot 11'),
        ('slot-in-io-sts', 'Input I/O status'),
        # TODO: Add more annotation classes:
        # TAG: 'ready', 'valid', 'id', 'rsv'
        # CMD ADDR: 'r/w', 'addr', 'unused'
        # CMD DATA: 'data', 'unused'
        # 3-11: 'data', 'unused', 'double data'
        ('warning', 'Warning'),
        ('error', 'Error'),
    )
    annotation_rows = (
        ('bits-out', 'Output bits', (Ann.BITS_OUT,)),
        ('slots-out-raw', 'Output numbers', (Ann.SLOT_OUT_RAW,)),
        ('slots-out', 'Output slots', (
            Ann.SLOT_OUT_TAG, Ann.SLOT_OUT_ADDR, Ann.SLOT_OUT_DATA,
            Ann.SLOT_OUT_03, Ann.SLOT_OUT_04, Ann.SLOT_OUT_05, Ann.SLOT_OUT_06,
            Ann.SLOT_OUT_07, Ann.SLOT_OUT_08, Ann.SLOT_OUT_09, Ann.SLOT_OUT_10,
            Ann.SLOT_OUT_11, Ann.SLOT_OUT_IO,)),
        ('bits-in', 'Input bits', (Ann.BITS_IN,)),
        ('slots-in-raw', 'Input numbers', (Ann.SLOT_IN_RAW,)),
        ('slots-in', 'Input slots', (
            Ann.SLOT_IN_TAG, Ann.SLOT_IN_ADDR, Ann.SLOT_IN_DATA,
            Ann.SLOT_IN_03, Ann.SLOT_IN_04, Ann.SLOT_IN_05, Ann.SLOT_IN_06,
            Ann.SLOT_IN_07, Ann.SLOT_IN_08, Ann.SLOT_IN_09, Ann.SLOT_IN_10,
            Ann.SLOT_IN_11, Ann.SLOT_IN_IO,)),
        ('warnings', 'Warnings', (Ann.WARN,)),
        ('errors', 'Errors', (Ann.ERROR,)),
    )
    binary = (
        ('frame-out', 'Frame bits, output data'),
        ('frame-in', 'Frame bits, input data'),
        ('slot-raw-out', 'Raw slot bits, output data'),
        ('slot-raw-in', 'Raw slot bits, input data'),
        # TODO: Which (other) binary classes to implement?
        # - Are binary annotations per audio slot useful?
        # - Assume 20bit per slot, in 24bit units? Or assume 16bit
        #   audio samples? Observe register access and derive width
        #   of the audio data? Dump channels 3-11 or 1-12?
    )

    def putx(self, ss, es, cls, data):
        self.put(ss, es, self.out_ann, [cls, data])

    def putf(self, frombit, bitcount, cls, data):
        ss = self.frame_ss_list[frombit]
        es = self.frame_ss_list[frombit + bitcount]
        self.putx(ss, es, cls, data)

    def putb(self, frombit, bitcount, cls, data):
        ss = self.frame_ss_list[frombit]
        es = self.frame_ss_list[frombit + bitcount]
        self.put(ss, es, self.out_binary, [cls, data])

    def __init__(self):
        self.reset()

    def reset(self):
        self.frame_ss_list = None
        self.frame_slot_lens = [0, 16] + [16 + 20 * i for i in range(1, 13)]
        self.frame_total_bits = self.frame_slot_lens[-1]
        self.handle_slots = {
            0: self.handle_slot_00,
            1: self.handle_slot_01,
            2: self.handle_slot_02,
        }

    def start(self):
        self.out_binary = self.register(srd.OUTPUT_BINARY)
        self.out_ann = self.register(srd.OUTPUT_ANN)

    def metadata(self, key, value):
        if key == srd.SRD_CONF_SAMPLERATE:
            self.samplerate = value

    def bits_to_int(self, bits):
        # Convert MSB-first bit sequence to integer value.
        if not bits:
            return 0
        count = len(bits)
        value = sum([2 ** (count - 1 - i) for i in range(count) if bits[i]])
        return value

    def bits_to_bin_ann(self, bits):
        # Convert MSB-first bit sequence to binary annotation data.
        # It's assumed that the number of bits does not (in useful ways)
        # fit into an integer, and we need to create an array of bytes
        # from the data afterwards, anyway. Hence the separate routine
        # and the conversion of eight bits each.
        out = []
        count = len(bits)
        while count > 0:
            count -= 8
            by, bits = bits[:8], bits[8:]
            by = self.bits_to_int(by)
            out.append(by)
        out = bytes(out)
        return out

    def int_to_nibble_text(self, value, bitcount):
        # Convert number to hex digits for given bit count.
        digits = (bitcount + 3) // 4
        text = '{{:0{:d}x}}'.format(digits).format(value)
        return text

    def get_bit_field(self, data, size, off, count):
        shift = size - off - count
        data >>= shift
        mask = (1 << count) - 1
        data &= mask
        return data

    def flush_frame_bits(self):
        # Flush raw frame bits to binary annotation.
        anncls = Ann.BIN_FRAME_OUT
        data = self.frame_bits_out[:]
        count = len(data)
        data = self.bits_to_bin_ann(data)
        self.putb(0, count, anncls, data)

        anncls = Ann.BIN_FRAME_IN
        data = self.frame_bits_in[:]
        count = len(data)
        data = self.bits_to_bin_ann(data)
        self.putb(0, count, anncls, data)

    def start_frame(self, ss):
        # Mark the start of a frame.
        if self.frame_ss_list:
            # Flush bits if we had a frame before the frame which is
            # starting here.
            self.flush_frame_bits()
        self.frame_ss_list = [ss]
        self.frame_bits_out = []
        self.frame_bits_in = []
        self.frame_slot_data_out = []
        self.frame_slot_data_in = []
        self.have_slots = {True: None, False: None}

    def handle_slot_dummy(self, slotidx, bitidx, bitcount, is_out, data):
        # Handle slot x, default/fallback handler.
        # Only process data of slots 1-12 when slot 0 says "valid".
        if not self.have_slots[is_out]:
            return
        if not self.have_slots[is_out][slotidx]:
            return

        # Emit a naive annotation with just the data bits that we saw
        # for the slot (hex nibbles for density). For audio data this
        # can be good enough. Slots with special meaning should not end
        # up calling the dummy handler.
        text = self.int_to_nibble_text(data, bitcount)
        anncls = Ann.SLOT_OUT_TAG if is_out else Ann.SLOT_IN_TAG
        self.putf(bitidx, bitcount, anncls + slotidx, [text])

        # Emit binary output for the data that is contained in slots
        # which end up calling the default handler. This transparently
        # should translate to "the slots with audio data", as other
        # slots which contain management data should have their specific
        # handler routines. In the present form, this approach might be
        # good enough to get a (header-less) audio stream for typical
        # setups where only line-in or line-out are in use.
        #
        # TODO: Improve this early prototype implementation. For now the
        # decoder just exports the upper 16 bits of each audio channel
        # that happens to be valid. For an improved implementation, it
        # either takes user provided specs or more smarts like observing
        # register access (if the capture includes it).
        anncls = Ann.BIN_SLOT_RAW_OUT if is_out else Ann.BIN_SLOT_RAW_IN
        data_bin = data >> 4
        data_bin &= 0xffff
        data_bin = data_bin.to_bytes(2, byteorder = 'big')
        self.putb(bitidx, bitcount, anncls, data_bin)

    def handle_slot_00(self, slotidx, bitidx, bitcount, is_out, data):
        # Handle slot 0, TAG.
        slotpos = self.frame_slot_lens[slotidx]
        fieldoff = 0
        anncls = Ann.SLOT_OUT_TAG if is_out else Ann.SLOT_IN_TAG

        fieldlen = 1
        ready = self.get_bit_field(data, bitcount, fieldoff, fieldlen)
        text = ['READY: 1', 'READY', 'RDY', 'R'] if ready else ['ready: 0', 'rdy', '-']
        self.putf(slotpos + fieldoff, fieldlen, anncls, text)
        fieldoff += fieldlen

        fieldlen = 12
        valid = self.get_bit_field(data, bitcount, fieldoff, fieldlen)
        text = ['VALID: {:3x}'.format(valid), '{:3x}'.format(valid)]
        self.putf(slotpos + fieldoff, fieldlen, anncls, text)
        have_slots = [True] + [False] * 12
        for idx in range(12):
            have_slots[idx + 1] = bool(valid & (1 << (11 - idx)))
        self.have_slots[is_out] = have_slots
        fieldoff += fieldlen

        fieldlen = 1
        rsv = self.get_bit_field(data, bitcount, fieldoff, fieldlen)
        if rsv != 0:
            text = ['reserved bit error', 'rsv error', 'rsv']
            self.putf(slotpos + fieldoff, fieldlen, Ann.ERROR, text)
        fieldoff += fieldlen

        # TODO: Will input slot 0 have a Codec ID, or 3 reserved bits?
        fieldlen = 2
        codec = self.get_bit_field(data, bitcount, fieldoff, fieldlen)
        text = ['CODEC: {:1x}'.format(codec), '{:1x}'.format(codec)]
        self.putf(slotpos + fieldoff, fieldlen, anncls, text)
        fieldoff += fieldlen

    def handle_slot_01(self, slotidx, bitidx, bitcount, is_out, data):
        # Handle slot 1, command/status address.
        slotpos = self.frame_slot_lens[slotidx]
        if not self.have_slots[is_out]:
            return
        if not self.have_slots[is_out][slotidx]:
            return
        fieldoff = 0
        anncls = Ann.SLOT_OUT_TAG if is_out else Ann.SLOT_IN_TAG
        anncls += slotidx

        fieldlen = 1
        if is_out:
            is_read = self.get_bit_field(data, bitcount, fieldoff, fieldlen)
            text = ['READ', 'RD', 'R'] if is_read else ['WRITE', 'WR', 'W']
            self.putf(slotpos + fieldoff, fieldlen, anncls, text)
            # TODO: Check for the "atomic" constraint? Some operations
            # involve address _and_ data, which cannot be spread across
            # several frames. Slot 0 and 1 _must_ be provided within the
            # same frame (the test should occur in the handler for slot
            # 2 of course, in slot 1 we don't know what will follow).
        else:
            rsv = self.get_bit_field(data, bitcount, fieldoff, fieldlen)
            if rsv != 0:
                text = ['reserved bit error', 'rsv error', 'rsv']
                self.putf(slotpos + fieldoff, fieldlen, Ann.ERROR, text)
        fieldoff += fieldlen

        fieldlen = 7
        regaddr = self.get_bit_field(data, bitcount, fieldoff, fieldlen)
        # TODO: Present 0-63 or 0-126 as the address of the 16bit register?
        text = ['ADDR: {:2x}'.format(regaddr), '{:2x}'.format(regaddr)]
        self.putf(slotpos + fieldoff, fieldlen, anncls, text)
        if regaddr & 0x01:
            text = ['odd register address', 'odd reg addr', 'odd addr', 'odd']
            self.putf(slotpos + fieldoff, fieldlen, Ann.ERROR, text)
        fieldoff += fieldlen

        # Strictly speaking there are 10 data request bits and 2 reserved
        # bits for input slots, and 12 reserved bits for output slots. We
        # test for 10 and 2 bits, to simplify the logic. Only in case of
        # non-zero reserved bits for outputs this will result in "a little
        # strange" an annotation. This is a cosmetic issue, we don't mind.
        fieldlen = 10
        reqdata = self.get_bit_field(data, bitcount, fieldoff, fieldlen)
        if is_out and reqdata != 0:
            text = ['reserved bit error', 'rsv error', 'rsv']
            self.putf(slotpos + fieldoff, fieldlen, Ann.ERROR, text)
        if not is_out:
            text = ['REQ: {:3x}'.format(reqdata), '{:3x}'.format(reqdata)]
            self.putf(slotpos + fieldoff, fieldlen, anncls, text)
        fieldoff += fieldlen

        fieldlen = 2
        rsv = self.get_bit_field(data, bitcount, fieldoff, fieldlen)
        if rsv != 0:
            text = ['reserved bit error', 'rsv error', 'rsv']
            self.putf(slotpos + fieldoff, fieldlen, Ann.ERROR, text)
        fieldoff += fieldlen

    def handle_slot_02(self, slotidx, bitidx, bitcount, is_out, data):
        # Handle slot 2, command/status data.
        slotpos = self.frame_slot_lens[slotidx]
        if not self.have_slots[is_out]:
            return
        if not self.have_slots[is_out][slotidx]:
            return
        fieldoff = 0
        anncls = Ann.SLOT_OUT_TAG if is_out else Ann.SLOT_IN_TAG
        anncls += slotidx

        fieldlen = 16
        rwdata = self.get_bit_field(data, bitcount, fieldoff, fieldlen)
        # TODO: Check for zero output data when the operation is a read.
        # TODO: Check for the "atomic" constraint.
        text = ['DATA: {:4x}'.format(rwdata), '{:4x}'.format(rwdata)]
        self.putf(slotpos + fieldoff, fieldlen, anncls, text)
        fieldoff += fieldlen

        fieldlen = 4
        rsv = self.get_bit_field(data, bitcount, fieldoff, fieldlen)
        if rsv != 0:
            text = ['reserved bits error', 'rsv error', 'rsv']
            self.putf(slotpos + fieldoff, fieldlen, Ann.ERROR, text)
        fieldoff += fieldlen

    # TODO: Implement other slots.
    # - 1: cmd/status addr (check status vs command)
    # - 2: cmd/status data (check status vs command)
    # - 3-11: audio out/in
    # - 12: io control/status (modem GPIO(?))

    def handle_slot(self, slotidx, data_out, data_in):
        # Process a received slot of a frame.
        func = self.handle_slots.get(slotidx, self.handle_slot_dummy)
        bitidx = self.frame_slot_lens[slotidx]
        bitcount = self.frame_slot_lens[slotidx + 1] - bitidx
        if data_out is not None:
            func(slotidx, bitidx, bitcount, True, data_out)
        if data_in is not None:
            func(slotidx, bitidx, bitcount, False, data_in)

    def handle_bits(self, ss, es, bit_out, bit_in):
        # Process a received pair of bits.
        # Emit the bits' annotations. Only interpret the data when we
        # are in a frame (have seen the start of the frame, and don't
        # exceed the expected number of bits in a frame).
        if bit_out is not None:
            self.putx(ss, es, Ann.BITS_OUT, ['{:d}'.format(bit_out)])
        if bit_in is not None:
            self.putx(ss, es, Ann.BITS_IN, ['{:d}'.format(bit_in)])
        if self.frame_ss_list is None:
            return
        self.frame_ss_list.append(es)
        have_len = len(self.frame_ss_list) - 1
        if have_len > self.frame_total_bits:
            return

        # Accumulate the bits within the frame, until one slot of the
        # frame has become available.
        slot_idx = 0
        if bit_out is not None:
            self.frame_bits_out.append(bit_out)
            slot_idx = len(self.frame_slot_data_out)
        if bit_in is not None:
            self.frame_bits_in.append(bit_in)
            slot_idx = len(self.frame_slot_data_in)
        want_len = self.frame_slot_lens[slot_idx + 1]
        if have_len != want_len:
            return
        prev_len = self.frame_slot_lens[slot_idx]

        # Convert bits to integer values. This shall simplify extraction
        # of bit fields in multiple other locations.
        slot_data_out = None
        if bit_out is not None:
            slot_bits = self.frame_bits_out[prev_len:]
            slot_data = self.bits_to_int(slot_bits)
            self.frame_slot_data_out.append(slot_data)
            slot_data_out = slot_data
        slot_data_in = None
        if bit_in is not None:
            slot_bits = self.frame_bits_in[prev_len:]
            slot_data = self.bits_to_int(slot_bits)
            self.frame_slot_data_in.append(slot_data)
            slot_data_in = slot_data

        # Emit simple annotations for the integer values, until upper
        # layer decode stages will be implemented.
        slot_len = have_len - prev_len
        slot_ss = self.frame_ss_list[prev_len]
        slot_es = self.frame_ss_list[have_len]
        if slot_data_out is not None:
            slot_text = self.int_to_nibble_text(slot_data_out, slot_len)
            self.putx(slot_ss, slot_es, Ann.SLOT_OUT_RAW, [slot_text])
        if slot_data_in is not None:
            slot_text = self.int_to_nibble_text(slot_data_in, slot_len)
            self.putx(slot_ss, slot_es, Ann.SLOT_IN_RAW, [slot_text])

        self.handle_slot(slot_idx, slot_data_out, slot_data_in)

    def decode(self):
        have_sdo = self.has_channel(Pins.SDATA_OUT)
        have_sdi = self.has_channel(Pins.SDATA_IN)
        if not have_sdo and not have_sdi:
            raise ChannelError('Either SDATA_OUT or SDATA_IN (or both) are required.')
        have_reset = self.has_channel(Pins.RESET)

        # Data is sampled at falling CLK edges. Annotations need to span
        # the period between rising edges. SYNC rises one cycle _before_
        # the start of a frame. Grab the earliest SYNC sample we can get
        # and advance to the start of a bit time. Then keep getting the
        # samples and the end of all subsequent bit times.
        prev_sync = [None, None, None]
        pins = self.wait({Pins.BIT_CLK: 'e'})
        if pins[Pins.BIT_CLK] == 0:
            prev_sync[-1] = pins[Pins.SYNC]
            pins = self.wait({Pins.BIT_CLK: 'r'})
        bit_ss = self.samplenum
        while True:
            pins = self.wait({Pins.BIT_CLK: 'f'})
            prev_sync.pop(0)
            prev_sync.append(pins[Pins.SYNC])
            self.wait({Pins.BIT_CLK: 'r'})
            if prev_sync[0] == 0 and prev_sync[1] == 1:
                self.start_frame(bit_ss)
            self.handle_bits(bit_ss, self.samplenum,
                    pins[Pins.SDATA_OUT] if have_sdo else None,
                    pins[Pins.SDATA_IN] if have_sdi else None)
            bit_ss = self.samplenum



##
## This file is part of the libsigrokdecode project.
##
## Copyright (C) 2015 Josef Gajdusek <atx@atx.name>
##
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, see <http://www.gnu.org/licenses/>.
##

import sigrokdecode as srd
import operator
import collections
from functools import reduce

end_codes = (
    'Unknown',
    'Charge Complete',
    'Internal Fault',
    'Over Temperature',
    'Over Voltage',
    'Over Current',
    'Battery Failure',
    'Reconfigure',
    'No Response',
)

class SamplerateError(Exception):
    pass

def calc_checksum(packet):
    return reduce(operator.xor, packet[:-1])

def bits_to_uint(bits):
    # LSB first
    return reduce(lambda i, v: (i >> 1) | (v << (len(bits) - 1)), bits, 0)

class Decoder(srd.Decoder):
    api_version = 3
    id = 'qi'
    name = 'Qi'
    longname = 'Qi charger protocol'
    desc = 'Protocol used by Qi receiver.'
    license = 'gplv2+'
    inputs = ['logic']
    outputs = []
    tags = ['Embedded/industrial', 'Wireless/RF']
    channels = (
        {'id': 'qi', 'name': 'Qi', 'desc': 'Demodulated Qi data line'},
    )
    annotations = (
        ('bits', 'Bits'),
        ('bytes-errors', 'Bit errors'),
        ('bytes-start', 'Start bits'),
        ('bytes-info', 'Info bits'),
        ('bytes-data', 'Data bytes'),
        ('packets-data', 'Packet data'),
        ('packets-checksum-ok', 'Packet checksum'),
        ('packets-checksum-err', 'Packet checksum'),
    )
    annotation_rows = (
        ('bits', 'Bits', (0,)),
        ('bytes', 'Bytes', (1, 2, 3, 4)),
        ('packets', 'Packets', (5, 6, 7)),
    )

    def __init__(self):
        self.reset()

    def reset(self):
        self.samplerate = None
        self.reset_variables()

    def reset_variables(self):
        self.counter = 0
        self.prev = None
        self.state = 'IDLE'
        self.lastbit = 0
        self.bytestart = 0
        self.deq = collections.deque(maxlen = 2)
        self.bits = []
        self.bitsi = [0]
        self.bytesi = []
        self.packet = []

    def metadata(self, key, value):
        if key == srd.SRD_CONF_SAMPLERATE:
            self.samplerate = value
            self.bit_width = float(self.samplerate) / 2e3

    def start(self):
        self.out_ann = self.register(srd.OUTPUT_ANN)
        self.reset_variables()

    def packet_len(self, byte):
        if 0x00 <= byte <= 0x1f:
            return int(1 + (byte - 0) / 32)
        if 0x20 <= byte <= 0x7f:
            return int(2 + (byte - 32) / 16)
        if 0x80 <= byte <= 0xdf:
            return int(8 + (byte - 128) / 8)
        if 0xe0 <= byte <= 0xff:
            return int(20 + (byte - 224) / 4)

    def in_tolerance(self, l):
        return (0.75 * self.bit_width) < l < (1.25 * self.bit_width)

    def putp(self, data):
        self.put(self.bytesi[0], self.bytesi[-1], self.out_ann, [5, data])

    def process_packet(self):
        if self.packet[0] == 0x01: # Signal Strength
            self.putp(['Signal Strength: %d' % self.packet[1],
                       'SS: %d' % self.packet[1], 'SS'])
        elif self.packet[0] == 0x02: # End Power Transfer
            reason = end_codes[self.packet[1]] if self.packet[1] < len(end_codes) else 'Reserved'
            self.putp(['End Power Transfer: %s' % reason,
                       'EPT: %s' % reason, 'EPT'])
        elif self.packet[0] == 0x03: # Control Error
            val = self.packet[1] if self.packet[1] < 128 else (self.packet[1] & 0x7f) - 128
            self.putp(['Control Error: %d' % val, 'CE: %d' % val, 'CE'])
        elif self.packet[0] == 0x04: # Received Power
            self.putp(['Received Power: %d' % self.packet[1],
                       'RP: %d' % self.packet[1], 'RP'])
        elif self.packet[0] == 0x05: # Charge Status
            self.putp(['Charge Status: %d' % self.packet[1],
                       'CS: %d' % self.packet[1], 'CS'])
        elif self.packet[0] == 0x06: # Power Control Hold-off
            self.putp(['Power Control Hold-off: %dms' % self.packet[1],
                       'PCH: %d' % self.packet[1]], 'PCH')
        elif self.packet[0] == 0x51: # Configuration
            powerclass = (self.packet[1] & 0xc0) >> 7
            maxpower = self.packet[1] & 0x3f
            prop = (self.packet[3] & 0x80) >> 7
            count = self.packet[3] & 0x07
            winsize = (self.packet[4] & 0xf8) >> 3
            winoff = self.packet[4] & 0x07
            self.putp(['Configuration: Power Class = %d, Maximum Power = %d, Prop = %d,'
                       'Count = %d, Window Size = %d, Window Offset = %d' %
                       (powerclass, maxpower, prop, count, winsize, winoff),
                       'C: PC = %d MP = %d P = %d C = %d WS = %d WO = %d' %
                       (powerclass, maxpower, prop, count, winsize, winoff),
                       'Configuration', 'C'])
        elif self.packet[0] == 0x71: # Identification
            version = '%d.%d' % ((self.packet[1] & 0xf0) >> 4, self.packet[1] & 0x0f)
            mancode = '%02x%02x' % (self.packet[2], self.packet[3])
            devid = '%02x%02x%02x%02x' % (self.packet[4] & ~0x80,
                    self.packet[5], self.packet[6], self.packet[7])
            self.putp(['Identification: Version = %s, Manufacturer = %s, ' \
                       'Device = %s' % (version, mancode, devid),
                       'ID: %s %s %s' % (version, mancode, devid), 'ID'])
        elif self.packet[0] == 0x81: # Extended Identification
            edevid = '%02x%02x%02x%02x%02x%02x%02x%02x' % self.packet[1:-1]
            self.putp(['Extended Identification: %s' % edevid,
                       'EI: %s' % edevid, 'EI'])
        elif self.packet[0] in (0x18, 0x19, 0x28, 0x29, 0x38, 0x48, 0x58, 0x68,
                0x78, 0x85, 0xa4, 0xc4, 0xe2): # Proprietary
            self.putp(['Proprietary', 'P'])
        else: # Unknown
            self.putp(['Unknown', '?'])
        self.put(self.bytesi[-1], self.samplenum, self.out_ann,
                 [6, ['Checksum OK', 'OK']] if \
                 calc_checksum(self.packet) == self.packet[-1]
                 else [6, ['Checksum error', 'ERR']])

    def process_byte(self):
        self.put(self.bytestart, self.bitsi[0], self.out_ann,
                 ([2, ['Start bit', 'Start', 'S']]) if self.bits[0] == 0 else
                 ([1, ['Start error', 'Start err', 'SE']]))
        databits = self.bits[1:9]
        data = bits_to_uint(databits)
        parity = reduce(lambda i, v: (i + v) % 2, databits, 1)
        self.put(self.bitsi[0], self.bitsi[8], self.out_ann, [4, ['%02x' % data]])
        self.put(self.bitsi[8], self.bitsi[9], self.out_ann,
                 ([3, ['Parity bit', 'Parity', 'P']]) if self.bits[9] == parity else
                 ([1, ['Parity error', 'Parity err', 'PE']]))
        self.put(self.bitsi[9], self.bitsi[10], self.out_ann,
                 ([3, ['Stop bit', 'Stop', 'S']]) if self.bits[10] == 1 else
                 ([1, ['Stop error', 'Stop err', 'SE']]))

        self.bytesi.append(self.bytestart)
        self.packet.append(data)
        if self.packet_len(self.packet[0]) + 2 == len(self.packet):
            self.process_packet()
            self.bytesi.clear()
            self.packet.clear()

    def add_bit(self, bit):
        self.bits.append(bit)
        self.bitsi.append(self.samplenum)

        if self.state == 'IDLE' and len(self.bits) >= 5 and \
                                    self.bits[-5:] == [1, 1, 1, 1, 0]:
            self.state = 'DATA'
            self.bytestart = self.bitsi[-2]
            self.bits = [0]
            self.bitsi = [self.samplenum]
            self.packet.clear()
        elif self.state == 'DATA' and len(self.bits) == 11:
            self.process_byte()
            self.bytestart = self.samplenum
            self.bits.clear()
            self.bitsi.clear()
        if self.state != 'IDLE':
            self.put(self.lastbit, self.samplenum, self.out_ann, [0, ['%d' % bit]])
        self.lastbit = self.samplenum

    def handle_transition(self, l, htl):
        self.deq.append(l)
        if len(self.deq) >= 2 and \
                (self.in_tolerance(self.deq[-1] + self.deq[-2]) or \
                htl and self.in_tolerance(l * 2) and \
                self.deq[-2] > 1.25 * self.bit_width):
            self.add_bit(1)
            self.deq.clear()
        elif self.in_tolerance(l):
            self.add_bit(0)
            self.deq.clear()
        elif l > (1.25 * self.bit_width):
            self.state = 'IDLE'
            self.bytesi.clear()
            self.packet.clear()
            self.bits.clear()
            self.bitsi.clear()

    def decode(self):
        if not self.samplerate:
            raise SamplerateError('Cannot decode without samplerate.')

        (qi,) = self.wait()
        self.handle_transition(self.samplenum, qi == 0)
        while True:
            prev = self.samplenum
            (qi,) = self.wait({0: 'e'})
            self.handle_transition(self.samplenum - prev, qi == 0)



##
## This file is part of the libsigrokdecode project.
##
## Copyright (C) 2013-2016 Uwe Hermann <uwe@hermann-uwe.de>
## Copyright (C) 2016 Chris Dreher <chrisdreher@hotmail.com>
##
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, see <http://www.gnu.org/licenses/>.
##

# Each status byte has 3 string names, each shorter than the previous
status_bytes = {
    # Channel voice messages
    0x80: ['note off', 'note off', 'N off'],
    0x90: ['note on', 'note on', 'N on'], # However, velocity = 0 means "note off".
    0xa0: ['polyphonic key pressure / aftertouch', 'key pressure', 'KP' ],
    0xb0: ['control change', 'ctrl chg', 'CC'],
    0xc0: ['program change', 'prgm chg', 'PC'],
    0xd0: ['channel pressure / aftertouch', 'channel pressure', 'CP'],
    0xe0: ['pitch bend change', 'pitch bend', 'PB'],

    # Channel mode messages
    # 0xb0: 'select channel mode', # Note: Same as 'control change'.

    # System exclusive messages
    0xf0: ['system exclusive', 'SysEx', 'SE'],

    # System common messages
    0xf1: ['MIDI time code quarter frame', 'MIDI time code', 'MIDI time'],
    0xf2: ['song position pointer', 'song position', 'song pos'],
    0xf3: ['song select', 'song select', 'song sel'],
    0xf4: ['undefined 0xf4', 'undef 0xf4', 'undef'],
    0xf5: ['undefined 0xf5', 'undef 0xf5', 'undef'],
    0xf6: ['tune request', 'tune request', 'tune req'],
    0xf7: ['end of system exclusive (EOX)', 'end of SysEx', 'EOX'],

    # System real time messages
    0xf8: ['timing clock', 'timing clock', 'clock'],
    0xf9: ['undefined 0xf9', 'undef 0xf9', 'undef'],
    0xfa: ['start', 'start', 's'],
    0xfb: ['continue', 'continue', 'cont'],
    0xfc: ['stop', 'stop', 'st'],
    0xfd: ['undefined 0xfd', 'undef 0xfd', 'undef'],
    0xfe: ['active sensing', 'active sensing', 'sensing'],
    0xff: ['system reset', 'reset', 'rst'],
}

# Universal system exclusive (SysEx) messages, non-realtime (0x7e)
universal_sysex_nonrealtime = {
    (0x00, None): 'unused',
    (0x01, None): 'sample dump header',
    (0x02, None): 'sample data packet',
    (0x03, None): 'sample dump request',

    (0x04, None): 'MIDI time code',
    (0x04, 0x00): 'special',
    (0x04, 0x01): 'punch in points',
    (0x04, 0x02): 'punch out points',
    (0x04, 0x03): 'delete punch in point',
    (0x04, 0x04): 'delete punch out point',
    (0x04, 0x05): 'event start point',
    (0x04, 0x06): 'event stop point',
    (0x04, 0x07): 'event start points with additional info',
    (0x04, 0x08): 'event stop points with additional info',
    (0x04, 0x09): 'delete event start point',
    (0x04, 0x0a): 'delete event stop point',
    (0x04, 0x0b): 'cue points',
    (0x04, 0x0c): 'cue points with additional info',
    (0x04, 0x0d): 'delete cue point',
    (0x04, 0x0e): 'event name in additional info',

    (0x05, None): 'sample dump extensions',
    (0x05, 0x01): 'multiple loop points',
    (0x05, 0x02): 'loop points request',

    (0x06, None): 'general information',
    (0x06, 0x01): 'identity request',
    (0x06, 0x02): 'identity reply',

    (0x07, None): 'file dump',
    (0x07, 0x01): 'header',
    (0x07, 0x02): 'data packet',
    (0x07, 0x03): 'request',

    (0x08, None): 'MIDI tuning standard',
    (0x08, 0x00): 'bulk dump request',
    (0x08, 0x01): 'bulk dump reply',

    (0x09, None): 'general MIDI',
    (0x09, 0x01): 'general MIDI system on',
    (0x09, 0x02): 'general MIDI system off',

    (0x7b, None): 'end of file',
    (0x7c, None): 'wait',
    (0x7d, None): 'cancel',
    (0x7e, None): 'nak',
    (0x7f, None): 'ack',
}

# Universal system exclusive (SysEx) messages, realtime (0x7f)
universal_sysex_realtime = {
    (0x00, None): 'unused',

    (0x01, None): 'MIDI time code',
    (0x01, 0x01): 'full message',
    (0x01, 0x02): 'user bits',

    (0x02, None): 'MIDI show control',
    (0x02, 0x00): 'MSC extensions',
    # (0x02, TODO): 'TODO', # 0x01 - 0x7f: MSC commands.

    (0x03, None): 'notation information',
    (0x03, 0x01): 'bar number',
    (0x03, 0x02): 'time signature (immediate)',
    (0x03, 0x42): 'time signature (delayed)',

    (0x04, None): 'device control',
    (0x04, 0x01): 'master volume',
    (0x04, 0x02): 'master balance',

    (0x05, None): 'real time MTC cueing',
    (0x05, 0x00): 'special',
    (0x05, 0x01): 'punch in points',
    (0x05, 0x02): 'punch out points',
    (0x05, 0x03): 'reserved',
    (0x05, 0x04): 'reserved',
    (0x05, 0x05): 'event start points',
    (0x05, 0x06): 'event stop points',
    (0x05, 0x07): 'event start points with additional info',
    (0x05, 0x08): 'event stop points with additional info',
    (0x05, 0x09): 'reserved',
    (0x05, 0x0a): 'reserved',
    (0x05, 0x0b): 'cue points',
    (0x05, 0x0c): 'cue points with additional info',
    (0x05, 0x0d): 'reserved',
    (0x05, 0x0e): 'event name in additional info',

    (0x06, None): 'MIDI machine control commands',
    # (0x06, TODO): 'TODO', # 0x00 - 0x7f: MMC commands.

    (0x07, None): 'MIDI machine control responses',
    # (0x07, TODO): 'TODO', # 0x00 - 0x7f: MMC commands.

    (0x08, None): 'MIDI tuning standard',
    (0x85, 0x02): 'note change',
}

# Note: Not all IDs are used/listed, i.e. there are some "holes".
sysex_manufacturer_ids = {
    # American group (range 01-1f, 000001-001f7f)
    (0x01,): 'Sequential',
    (0x02,): 'IDP',
    (0x03,): 'Voyetra/Octave-Plateau',
    (0x04,): 'Moog',
    (0x05,): 'Passport Designs',
    (0x06,): 'Lexicon',
    (0x07,): 'Kurzweil',
    (0x08,): 'Fender',
    (0x09,): 'Gulbransen',
    (0x0a,): 'AKG Acoustics',
    (0x0b,): 'Voyce Music',
    (0x0c,): 'Waveframe Corp',
    (0x0d,): 'ADA Signal Processors',
    (0x0e,): 'Garfield Electronics',
    (0x0f,): 'Ensoniq',
    (0x10,): 'Oberheim',
    (0x11,): 'Apple Computer',
    (0x12,): 'Grey Matter Response',
    (0x13,): 'Digidesign',
    (0x14,): 'Palm Tree Instruments',
    (0x15,): 'JLCooper Electronics',
    (0x16,): 'Lowrey',
    (0x17,): 'Adams-Smith',
    (0x18,): 'Emu Systems',
    (0x19,): 'Harmony Systems',
    (0x1a,): 'ART',
    (0x1b,): 'Baldwin',
    (0x1c,): 'Eventide',
    (0x1d,): 'Inventronics',
    (0x1f,): 'Clarity',

    (0x00, 0x00, 0x01): 'Time Warner Interactive',
    (0x00, 0x00, 0x07): 'Digital Music Corp.',
    (0x00, 0x00, 0x08): 'IOTA Systems',
    (0x00, 0x00, 0x09): 'New England Digital',
    (0x00, 0x00, 0x0a): 'Artisyn',
    (0x00, 0x00, 0x0b): 'IVL Technologies',
    (0x00, 0x00, 0x0c): 'Southern Music Systems',
    (0x00, 0x00, 0x0d): 'Lake Butler Sound Company',
    (0x00, 0x00, 0x0e): 'Alesis',
    (0x00, 0x00, 0x10): 'DOD Electronics',
    (0x00, 0x00, 0x11): 'Studer-Editech',
    (0x00, 0x00, 0x14): 'Perfect Fretworks',
    (0x00, 0x00, 0x15): 'KAT',
    (0x00, 0x00, 0x16): 'Opcode',
    (0x00, 0x00, 0x17): 'Rane Corp.',
    (0x00, 0x00, 0x18): 'Anadi Inc.',
    (0x00, 0x00, 0x19): 'KMX',
    (0x00, 0x00, 0x1a): 'Allen & Heath Brenell',
    (0x00, 0x00, 0x1b): 'Peavy Electronics',
    (0x00, 0x00, 0x1c): '360 Systems',
    (0x00, 0x00, 0x1d): 'Spectrum Design and Development',
    (0x00, 0x00, 0x1e): 'Marquis Music',
    (0x00, 0x00, 0x1f): 'Zeta Systems',

    (0x00, 0x00, 0x20): 'Axxes',
    (0x00, 0x00, 0x21): 'Orban',
    (0x00, 0x00, 0x24): 'KTI',
    (0x00, 0x00, 0x25): 'Breakaway Technologies',
    (0x00, 0x00, 0x26): 'CAE',
    (0x00, 0x00, 0x29): 'Rocktron Corp.',
    (0x00, 0x00, 0x2a): 'PianoDisc',
    (0x00, 0x00, 0x2b): 'Cannon Research Group',
    (0x00, 0x00, 0x2d): 'Rogers Instrument Corp.',
    (0x00, 0x00, 0x2e): 'Blue Sky Logic',
    (0x00, 0x00, 0x2f): 'Encore Electronics',

    (0x00, 0x00, 0x30): 'Uptown',
    (0x00, 0x00, 0x31): 'Voce',
    (0x00, 0x00, 0x32): 'CTI Audio, Inc. (Music. Intel Dev.)',
    (0x00, 0x00, 0x33): 'S&S Research',
    (0x00, 0x00, 0x34): 'Broderbund Software, Inc.',
    (0x00, 0x00, 0x35): 'Allen Organ Co.',
    (0x00, 0x00, 0x37): 'Music Quest',
    (0x00, 0x00, 0x38): 'APHEX',
    (0x00, 0x00, 0x39): 'Gallien Krueger',
    (0x00, 0x00, 0x3a): 'IBM',
    (0x00, 0x00, 0x3c): 'Hotz Instruments Technologies',
    (0x00, 0x00, 0x3d): 'ETA Lighting',
    (0x00, 0x00, 0x3e): 'NSI Corporation',
    (0x00, 0x00, 0x3f): 'Ad Lib, Inc.',

    (0x00, 0x00, 0x40): 'Richmond Sound Design',
    (0x00, 0x00, 0x41): 'Microsoft',
    (0x00, 0x00, 0x42): 'The Software Toolworks',
    (0x00, 0x00, 0x43): 'Niche/RJMG',
    (0x00, 0x00, 0x44): 'Intone',
    (0x00, 0x00, 0x47): 'GT Electronics / Groove Tubes',
    (0x00, 0x00, 0x49): 'Timeline Vista',
    (0x00, 0x00, 0x4a): 'Mesa Boogie',
    (0x00, 0x00, 0x4c): 'Sequoia Development',
    (0x00, 0x00, 0x4d): 'Studio Electronics',
    (0x00, 0x00, 0x4e): 'Euphonix',
    (0x00, 0x00, 0x4f): 'InterMIDI, Inc.',

    (0x00, 0x00, 0x50): 'MIDI Solutions',
    (0x00, 0x00, 0x51): '3DO Company',
    (0x00, 0x00, 0x52): 'Lightwave Research',
    (0x00, 0x00, 0x53): 'Micro-W',
    (0x00, 0x00, 0x54): 'Spectral Synthesis',
    (0x00, 0x00, 0x55): 'Lone Wolf',
    (0x00, 0x00, 0x56): 'Studio Technologies',
    (0x00, 0x00, 0x57): 'Peterson EMP',
    (0x00, 0x00, 0x58): 'Atari',
    (0x00, 0x00, 0x59): 'Marion Systems',
    (0x00, 0x00, 0x5a): 'Design Event',
    (0x00, 0x00, 0x5b): 'Winjammer Software',
    (0x00, 0x00, 0x5c): 'AT&T Bell Labs',
    (0x00, 0x00, 0x5e): 'Symetrix',
    (0x00, 0x00, 0x5f): 'MIDI the World',

    (0x00, 0x00, 0x60): 'Desper Products',
    (0x00, 0x00, 0x61): 'Micros\'N MIDI',
    (0x00, 0x00, 0x62): 'Accordians Intl',
    (0x00, 0x00, 0x63): 'EuPhonics',
    (0x00, 0x00, 0x64): 'Musonix',
    (0x00, 0x00, 0x65): 'Turtle Beach Systems',
    (0x00, 0x00, 0x66): 'Mackie Designs',
    (0x00, 0x00, 0x67): 'Compuserve',
    (0x00, 0x00, 0x68): 'BES Technologies',
    (0x00, 0x00, 0x69): 'QRS Music Rolls',
    (0x00, 0x00, 0x6a): 'P G Music',
    (0x00, 0x00, 0x6b): 'Sierra Semiconductor',
    (0x00, 0x00, 0x6c): 'EpiGraf Audio Visual',
    (0x00, 0x00, 0x6d): 'Electronics Deiversified',
    (0x00, 0x00, 0x6e): 'Tune 1000',
    (0x00, 0x00, 0x6f): 'Advanced Micro Devices',

    (0x00, 0x00, 0x70): 'Mediamation',
    (0x00, 0x00, 0x71): 'Sabine Music',
    (0x00, 0x00, 0x72): 'Woog Labs',
    (0x00, 0x00, 0x73): 'Micropolis',
    (0x00, 0x00, 0x74): 'Ta Horng Musical Inst.',
    (0x00, 0x00, 0x75): 'eTek (formerly Forte)',
    (0x00, 0x00, 0x76): 'Electrovoice',
    (0x00, 0x00, 0x77): 'Midisoft',
    (0x00, 0x00, 0x78): 'Q-Sound Labs',
    (0x00, 0x00, 0x79): 'Westrex',
    (0x00, 0x00, 0x7a): 'NVidia',
    (0x00, 0x00, 0x7b): 'ESS Technology',
    (0x00, 0x00, 0x7c): 'MediaTrix Peripherals',
    (0x00, 0x00, 0x7d): 'Brooktree',
    (0x00, 0x00, 0x7e): 'Otari',
    (0x00, 0x00, 0x7f): 'Key Electronics',

    (0x00, 0x01, 0x01): 'Crystalake Multimedia',
    (0x00, 0x01, 0x02): 'Crystal Semiconductor',
    (0x00, 0x01, 0x03): 'Rockwell Semiconductor',

    # European group (range 20-3f, 002000-003f7f)
    (0x20,): 'Passac',
    (0x21,): 'SIEL',
    (0x22,): 'Synthaxe',
    (0x24,): 'Hohner',
    (0x25,): 'Twister',
    (0x26,): 'Solton',
    (0x27,): 'Jellinghaus MS',
    (0x28,): 'Southworth Music Systems',
    (0x29,): 'PPG',
    (0x2a,): 'JEN',
    (0x2b,): 'SSL Limited',
    (0x2c,): 'Audio Veritrieb',
    (0x2f,): 'Elka',

    (0x30,): 'Dynacord',
    (0x31,): 'Viscount',
    (0x33,): 'Clavia Digital Instruments',
    (0x34,): 'Audio Architecture',
    (0x35,): 'GeneralMusic Corp.',
    (0x39,): 'Soundcraft Electronics',
    (0x3b,): 'Wersi',
    (0x3c,): 'Avab Elektronik Ab',
    (0x3d,): 'Digigram',
    (0x3e,): 'Waldorf Electronics',
    (0x3f,): 'Quasimidi',

    (0x00, 0x20, 0x00): 'Dream',
    (0x00, 0x20, 0x01): 'Strand Lighting',
    (0x00, 0x20, 0x02): 'Amek Systems',
    (0x00, 0x20, 0x04): 'Böhm Electronic',
    (0x00, 0x20, 0x06): 'Trident Audio',
    (0x00, 0x20, 0x07): 'Real World Studio',
    (0x00, 0x20, 0x09): 'Yes Technology',
    (0x00, 0x20, 0x0a): 'Audiomatica',
    (0x00, 0x20, 0x0b): 'Bontempi/Farfisa',
    (0x00, 0x20, 0x0c): 'F.B.T. Elettronica',
    (0x00, 0x20, 0x0d): 'MidiTemp',
    (0x00, 0x20, 0x0e): 'LA Audio (Larking Audio)',
    (0x00, 0x20, 0x0f): 'Zero 88 Lighting Limited',

    (0x00, 0x20, 0x10): 'Micon Audio Electronics GmbH',
    (0x00, 0x20, 0x11): 'Forefront Technology',
    (0x00, 0x20, 0x13): 'Kenton Electronics',
    (0x00, 0x20, 0x15): 'ADB',
    (0x00, 0x20, 0x16): 'Marshall Products',
    (0x00, 0x20, 0x17): 'DDA',
    (0x00, 0x20, 0x18): 'BSS',
    (0x00, 0x20, 0x19): 'MA Lighting Technology',
    (0x00, 0x20, 0x1a): 'Fatar',
    (0x00, 0x20, 0x1b): 'QSC Audio',
    (0x00, 0x20, 0x1c): 'Artisan Classic Organ',
    (0x00, 0x20, 0x1d): 'Orla Spa',
    (0x00, 0x20, 0x1e): 'Pinnacle Audio',
    (0x00, 0x20, 0x1f): 'TC Electronics',

    (0x00, 0x20, 0x20): 'Doepfer Musikelektronik',
    (0x00, 0x20, 0x21): 'Creative Technology Pte',
    (0x00, 0x20, 0x22): 'Minami/Seiyddo',
    (0x00, 0x20, 0x23): 'Goldstar',
    (0x00, 0x20, 0x24): 'Midisoft s.a.s di M. Cima',
    (0x00, 0x20, 0x25): 'Samick',
    (0x00, 0x20, 0x26): 'Penny and Giles',
    (0x00, 0x20, 0x27): 'Acorn Computer',
    (0x00, 0x20, 0x28): 'LSC Electronics',
    (0x00, 0x20, 0x29): 'Novation EMS',
    (0x00, 0x20, 0x2a): 'Samkyung Mechatronics',
    (0x00, 0x20, 0x2b): 'Medeli Electronics',
    (0x00, 0x20, 0x2c): 'Charlie Lab',
    (0x00, 0x20, 0x2d): 'Blue Chip Music Tech',
    (0x00, 0x20, 0x2e): 'BEE OH Corp',

    # Japanese group (range 40-5f, 004000-005f7f)
    (0x40,): 'Kawai',
    (0x41,): 'Roland',
    (0x42,): 'Korg',
    (0x43,): 'Yamaha',
    (0x44,): 'Casio',
    (0x46,): 'Kamiya Studio',
    (0x47,): 'Akai',
    (0x48,): 'Japan Victor',
    (0x49,): 'Mesosha',
    (0x4a,): 'Hoshino Gakki',
    (0x4b,): 'Fujitsu Elect',
    (0x4c,): 'Sony',
    (0x4d,): 'Nisshin Onpa',
    (0x4e,): 'TEAC',
    (0x50,): 'Matsushita Electric',
    (0x51,): 'Fostex',
    (0x52,): 'Zoom',
    (0x53,): 'Midori Electronics',
    (0x54,): 'Matsushita Communication Industrial',
    (0x55,): 'Suzuki Musical Inst. Mfg.',

    # Other (range 60-7c, 006000-007f7f)

    # Special (7d-7f)
    (0x7d,): 'Non-Commercial',
    (0x7e,): 'Universal Non-Realtime',
    (0x7f,): 'Universal Realtime',
}

control_functions = {
    0x00: ['bank select MSB', 'bank MSB', 'bank-M'],
    0x01: ['modulation wheel/lever MSB', 'modulation MSB', 'mod-M'],
    0x02: ['breath controller MSB', 'breath MSB', 'breath-M'],
    # 0x03: undefined MSB
    0x04: ['foot controller MSB', 'foot MSB', 'foot-M'],
    0x05: ['portamento time MSB', 'portamento MSB', 'porta-M'],
    0x06: ['data entry MSB', 'data entry MSB', 'data-M'],
    0x07: ['channel volume MSB (formerly main volume)', 'channel volume MSB', 'ch vol-M'],
    0x08: ['balance MSB', 'bal MSB', 'bal-M'],
    # 0x09: undefined MSB
    0x0a: ['pan MSB', 'pan MSB', 'pan-M'],
    0x0b: ['expression controller MSB', 'expression MSB', 'expr-M'],
    0x0c: ['effect control 1 MSB', 'effect 1 MSB', 'eff-1-M'],
    0x0d: ['effect control 2 MSB', 'effect 2 MSB', 'eff-2-M'],
    # 0x0e-0x0f: undefined MSB
    0x10: ['general purpose controller 1 MSB', 'GP ctrl 1 MSB', 'GPC-1-M'],
    0x11: ['general purpose controller 2 MSB', 'GP ctrl 2 MSB', 'GPC-2-M'],
    0x12: ['general purpose controller 3 MSB', 'GP ctrl 3 MSB', 'GPC-3-M'],
    0x13: ['general purpose controller 4 MSB', 'GP ctrl 4 MSB', 'GPC-4-M'],
    # 0x14-0x1f: undefined MSB
    0x20: ['bank select LSB', 'bank LSB', 'bank-L'],
    0x21: ['modulation wheel/lever LSB', 'modulation LSB', 'mod-L'],
    0x22: ['breath controller LSB', 'breath LSB', 'breath-L'],
    # 0x23: undefined LSB
    0x24: ['foot controller LSB', 'foot LSB', 'foot-L'],
    0x25: ['portamento time LSB', 'portamento LSB', 'porta-L'],
    0x26: ['data entry LSB', 'data entry LSB', 'data-L'],
    0x27: ['channel volume LSB (formerly main volume)', 'channel volume LSB', 'ch vol-L'],
    0x28: ['balance LSB', 'bal LSB', 'bal-L'],
    # 0x29: undefined LSB
    0x2a: ['pan LSB', 'pan LSB', 'pan-L'],
    0x2b: ['expression controller LSB', 'expression LSB', 'expr-L'],
    0x2c: ['effect control 1 LSB', 'effect 1 LSB', 'eff-1-L'],
    0x2d: ['effect control 2 LSB', 'effect 2 LSB', 'eff-2-L'],
    # 0x2e-0x2f: undefined LSB
    0x30: ['general purpose controller 1 LSB', 'GP ctrl 1 LSB', 'GPC-1-L'],
    0x31: ['general purpose controller 2 LSB', 'GP ctrl 2 LSB', 'GPC-2-L'],
    0x32: ['general purpose controller 3 LSB', 'GP ctrl 3 LSB', 'GPC-3-L'],
    0x33: ['general purpose controller 4 LSB', 'GP ctrl 4 LSB', 'GPC-4-L'],
    # 0x34-0x3f: undefined LSB
    0x40: ['damper pedal (sustain)', 'sustain', 'sust'],
    0x41: ['portamento on/off', 'porta on/off', 'porta on/off'],
    0x42: ['sostenuto', 'sostenuto', 'sostenuto'],
    0x43: ['soft pedal', 'soft pedal', 'soft pedal'],
    0x44: ['legato footswitch', 'legato switch', 'legato'], # vv: 00-3f = normal, 40-7f = legato
    0x45: ['hold 2', 'hold 2', 'hold 2'],
    0x46: ['sound controller 1 (default: sound variation)', 'sound ctrl 1', 'snd ctrl 1'],
    0x47: ['sound controller 2 (default: timbre / harmonic intensity)', 'sound ctrl 2', 'snd ctrl 2'],
    0x48: ['sound controller 3 (default: release time)', 'sound ctrl 3', 'snd ctrl 3'],
    0x49: ['sound controller 4 (default: attack time)', 'sound ctrl 4', 'snd ctrl 4'],
    0x4a: ['sound controller 5 (default: brightness)', 'sound ctrl 5', 'snd ctrl 5'],
    0x4b: ['sound controller 6 (GM2 default: decay time)', 'sound ctrl 6', 'snd ctrl 6'],
    0x4c: ['sound controller 7 (GM2 default: vibrato rate)', 'sound ctrl 7', 'snd ctrl 7'],
    0x4d: ['sound controller 8 (GM2 default: vibrato depth)', 'sound ctrl 8', 'snd ctrl 8'],
    0x4e: ['sound controller 9 (GM2 default: vibrato delay)', 'sound ctrl 9', 'snd ctrl 9'],
    0x4f: ['sound controller 10', 'sound ctrl 10', 'snd ctrl 10'],
    0x50: ['general purpose controller 5', 'GP controller 5', 'GPC-5'],
    0x51: ['general purpose controller 6', 'GP controller 6', 'GPC-6'],
    0x52: ['general purpose controller 7', 'GP controller 7', 'GPC-7'],
    0x53: ['general purpose controller 8', 'GP controller 8', 'GPC-8'],
    0x54: ['portamento control', 'portamento ctrl', 'porta ctrl'],
    # 0x55-0x5a: undefined
    0x5b: ['effects 1 depth (formerly external effects depth)', 'effects 1 depth', 'eff 1 depth'],
    0x5c: ['effects 2 depth (formerly tremolo depth)', 'effects 2 depth', 'eff 2 depth'],
    0x5d: ['effects 3 depth (formerly chorus depth)', 'effects 3 depth', 'eff 3 depth'],
    0x5e: ['effects 4 depth (formerly celeste/detune depth)', 'effects 4 depth', 'eff 4 depth'],
    0x5f: ['effects 5 depth (formerly phaser depth)', 'effects 5 depth', 'eff 5 depth'],
    0x60: ['data increment', 'data inc', 'data++'],
    0x61: ['data decrement', 'data dec', 'data--'],
    0x62: ['Non-Registered Parameter Number LSB', 'NRPN LSB', 'NRPN-L'],
    0x63: ['Non-Registered Parameter Number MSB', 'NRPN MSB', 'NRPN-M'],
    0x64: ['Registered Parameter Number LSB', 'RPN LSB', 'RPN-L'],
    0x65: ['Registered Parameter Number MSB', 'RPN MSB', 'RPN-M'],
    # 0x66-0x77: undefined
    # 0x78-0x7f: reserved for channel mode messages
    0x78: ['all sound off', 'all snd off', 'snd off'],
    0x79: ['reset all controllers', 'reset all ctrls', 'reset ctrls'],
    0x7a: ['local control', 'local ctrl', 'local ctrl'],
    0x7b: ['all notes off', 'notes off', 'notes off'],
    0x7c: ['omni mode off', 'omni off', 'omni off'], # all notes off
    0x7d: ['omni mode on', 'omni on', 'omni on'], # all notes off
    0x7e: ['mono mode on', 'mono on', 'mono'], # mono mode on, all notes off
    0x7f: ['poly mode on', 'poly on', 'poly'], # mono mode off, all notes off
}

gm_instruments = {
    1:   'Acoustic Grand Piano',
    2:   'Bright Acoustic Piano',
    3:   'Electric Grand Piano',
    4:   'Honky-tonk Piano',
    5:   'Electric Piano 1',
    6:   'Electric Piano 2',
    7:   'Harpsichord',
    8:   'Clavi',
    9:   'Celesta',
    10:  'Glockenspiel',
    11:  'Music Box',
    12:  'Vibraphone',
    13:  'Marimba',
    14:  'Xylophone',
    15:  'Tubular Bells',
    16:  'Dulcimer',
    17:  'Drawbar Organ',
    18:  'Percussive Organ',
    19:  'Rock Organ',
    20:  'Church Organ',
    21:  'Reed Organ',
    22:  'Accordion',
    23:  'Harmonica',
    24:  'Tango Accordion',
    25:  'Acoustic Guitar (nylon)',
    26:  'Acoustic Guitar (steel)',
    27:  'Electric Guitar (jazz)',
    28:  'Electric Guitar (clean)',
    29:  'Electric Guitar (muted)',
    30:  'Overdriven Guitar',
    31:  'Distortion Guitar',
    32:  'Guitar harmonics',
    33:  'Acoustic Bass',
    34:  'Electric Bass (finger)',
    35:  'Electric Bass (pick)',
    36:  'Fretless Bass',
    37:  'Slap Bass 1',
    38:  'Slap Bass 2',
    39:  'Synth Bass 1',
    40:  'Synth Bass 2',
    41:  'Violin',
    42:  'Viola',
    43:  'Cello',
    44:  'Contrabass',
    45:  'Tremolo Strings',
    46:  'Pizzicato Strings',
    47:  'Orchestral Harp',
    48:  'Timpani',
    49:  'String Ensemble 1',
    50:  'String Ensemble 2',
    51:  'SynthStrings 1',
    52:  'SynthStrings 2',
    53:  'Choir Aahs',
    54:  'Voice Oohs',
    55:  'Synth Voice',
    56:  'Orchestra Hit',
    57:  'Trumpet',
    58:  'Trombone',
    59:  'Tuba',
    60:  'Muted Trumpet',
    61:  'French Horn',
    62:  'Brass Section',
    63:  'SynthBrass 1',
    64:  'SynthBrass 2',
    65:  'Soprano Sax',
    66:  'Alto Sax',
    67:  'Tenor Sax',
    68:  'Baritone Sax',
    69:  'Oboe',
    70:  'English Horn',
    71:  'Bassoon',
    72:  'Clarinet',
    73:  'Piccolo',
    74:  'Flute',
    75:  'Recorder',
    76:  'Pan Flute',
    77:  'Blown Bottle',
    78:  'Shakuhachi',
    79:  'Whistle',
    80:  'Ocarina',
    81:  'Lead 1 (square)',
    82:  'Lead 2 (sawtooth)',
    83:  'Lead 3 (calliope)',
    84:  'Lead 4 (chiff)',
    85:  'Lead 5 (charang)',
    86:  'Lead 6 (voice)',
    87:  'Lead 7 (fifths)',
    88:  'Lead 8 (bass + lead)',
    89:  'Pad 1 (new age)',
    90:  'Pad 2 (warm)',
    91:  'Pad 3 (polysynth)',
    92:  'Pad 4 (choir)',
    93:  'Pad 5 (bowed)',
    94:  'Pad 6 (metallic)',
    95:  'Pad 7 (halo)',
    96:  'Pad 8 (sweep)',
    97:  'FX 1 (rain)',
    98:  'FX 2 (soundtrack)',
    99:  'FX 3 (crystal)',
    100: 'FX 4 (atmosphere)',
    101: 'FX 5 (brightness)',
    102: 'FX 6 (goblins)',
    103: 'FX 7 (echoes)',
    104: 'FX 8 (sci-fi)',
    105: 'Sitar',
    106: 'Banjo',
    107: 'Shamisen',
    108: 'Koto',
    109: 'Kalimba',
    110: 'Bag pipe',
    111: 'Fiddle',
    112: 'Shanai',
    113: 'Tinkle Bell',
    114: 'Agogo',
    115: 'Steel Drums',
    116: 'Woodblock',
    117: 'Taiko Drum',
    118: 'Melodic Tom',
    119: 'Synth Drum',
    120: 'Reverse Cymbal',
    121: 'Guitar Fret Noise',
    122: 'Breath Noise',
    123: 'Seashore',
    124: 'Bird Tweet',
    125: 'Telephone Ring',
    126: 'Helicopter',
    127: 'Applause',
    128: 'Gunshot',
}

drum_kit = {
    1:   'GM Standard Kit',
    9:   'GS Room Kit',
    17:  'GS Power Kit',
    25:  'GS Power Kit',
    26:  'GS TR-808 Kit',
    33:  'GS Jazz Kit',
    41:  'GS Brush Kit',
    49:  'GS Orchestra Kit',
    57:  'GS Sound FX Kit',
    128: 'GS CM-64/CM-32 Kit',
}

# Each quarter frame type has 2 string names, each shorter than the previous
quarter_frame_type = {
    0: ['frame count LS nibble', 'frame LSN'],
    1: ['frame count MS nibble', 'frame MSN'],
    2: ['seconds LS nibble', 'sec LSN'],
    3: ['seconds MS nibble', 'sec MSN'],
    4: ['minutes LS nibble', 'min LSN'],
    5: ['minutes MS nibble', 'min MSN'],
    6: ['hours LS nibble', 'hrs LSN'],
    7: ['hours MS nibble and SMPTE type', 'hrs MSN'],
}

smpte_type = {
    0: '24 fps',
    1: '25 fps',
    2: '30 fps (drop-frame)',
    3: '30 fps (non-drop)',
}

chromatic_notes = {
    0: 'C-1',
    1: 'C#-1',
    2: 'D-1',
    3: 'D#-1',
    4: 'E-1',
    5: 'F-1',
    6: 'F#-1',
    7: 'G-1',
    8: 'G#-1',
    9: 'A-1',
    10: 'A#-1',
    11: 'B-1',
    12: 'C0',
    13: 'C#0',
    14: 'D0',
    15: 'D#0',
    16: 'E0',
    17: 'F0',
    18: 'F#0',
    19: 'G0',
    20: 'G#0',
    21: 'A0',
    22: 'A#0',
    23: 'B0',
    24: 'C1',
    25: 'C#1',
    26: 'D1',
    27: 'D#1',
    28: 'E1',
    29: 'F1',
    30: 'F#1',
    31: 'G1',
    32: 'G#1',
    33: 'A1',
    34: 'A#1',
    35: 'B1',
    36: 'C2',
    37: 'C#2',
    38: 'D2',
    39: 'D#2',
    40: 'E2',
    41: 'F2',
    42: 'F#2',
    43: 'G2',
    44: 'G#2',
    45: 'A2',
    46: 'A#2',
    47: 'B2',
    48: 'C3',
    49: 'C#3',
    50: 'D3',
    51: 'D#3',
    52: 'E3',
    53: 'F3',
    54: 'F#3',
    55: 'G3',
    56: 'G#3',
    57: 'A3',
    58: 'A#3',
    59: 'B3',
    60: 'C4',
    61: 'C#4',
    62: 'D4',
    63: 'D#4',
    64: 'E4',
    65: 'F4',
    66: 'F#4',
    67: 'G4',
    68: 'G#4',
    69: 'A4',
    70: 'A#4',
    71: 'B4',
    72: 'C5',
    73: 'C#5',
    74: 'D5',
    75: 'D#5',
    76: 'E5',
    77: 'F5',
    78: 'F#5',
    79: 'G5',
    80: 'G#5',
    81: 'A5',
    82: 'A#5',
    83: 'B5',
    84: 'C6',
    85: 'C#6',
    86: 'D6',
    87: 'D#6',
    88: 'E6',
    89: 'F6',
    90: 'F#6',
    91: 'G6',
    92: 'G#6',
    93: 'A6',
    94: 'A#6',
    95: 'B6',
    96: 'C7',
    97: 'C#7',
    98: 'D7',
    99: 'D#7',
    100: 'E7',
    101: 'F7',
    102: 'F#7',
    103: 'G7',
    104: 'G#7',
    105: 'A7',
    106: 'A#7',
    107: 'B7',
    108: 'C8',
    109: 'C#8',
    110: 'D8',
    111: 'D#8',
    112: 'E8',
    113: 'F8',
    114: 'F#8',
    115: 'G8',
    116: 'G#8',
    117: 'A8',
    118: 'A#8',
    119: 'B8',
    120: 'C9',
    121: 'C#9',
    122: 'D9',
    123: 'D#9',
    124: 'E9',
    125: 'F9',
    126: 'F#9',
    127: 'G9',
}

percussion_notes = {
    35: 'Acoustic Bass Drum',
    36: 'Bass Drum 1',
    37: 'Side Stick',
    38: 'Acoustic Snare',
    39: 'Hand Clap',
    40: 'Electric Snare',
    41: 'Low Floor Tom',
    42: 'Closed Hi Hat',
    43: 'High Floor Tom',
    44: 'Pedal Hi-Hat',
    45: 'Low Tom',
    46: 'Open Hi-Hat',
    47: 'Low-Mid Tom',
    48: 'Hi Mid Tom',
    49: 'Crash Cymbal 1',
    50: 'High Tom',
    51: 'Ride Cymbal 1',
    52: 'Chinese Cymbal',
    53: 'Ride Bell',
    54: 'Tambourine',
    55: 'Splash Cymbal',
    56: 'Cowbell',
    57: 'Crash Cymbal 2',
    58: 'Vibraslap',
    59: 'Ride Cymbal 2',
    60: 'Hi Bongo',
    61: 'Low Bongo',
    62: 'Mute Hi Conga',
    63: 'Open Hi Conga',
    64: 'Low Conga',
    65: 'High Timbale',
    66: 'Low Timbale',
    67: 'High Agogo',
    68: 'Low Agogo',
    69: 'Cabasa',
    70: 'Maracas',
    71: 'Short Whistle',
    72: 'Long Whistle',
    73: 'Short Guiro',
    74: 'Long Guiro',
    75: 'Claves',
    76: 'Hi Wood Block',
    77: 'Low Wood Block',
    78: 'Mute Cuica',
    79: 'Open Cuica',
    80: 'Mute Triangle',
    81: 'Open Triangle',
}



##
## This file is part of the libsigrokdecode project.
##
## Copyright (C) 2012 Iztok Jeras <iztok.jeras@gmail.com>
##
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, see <http://www.gnu.org/licenses/>.
##

import sigrokdecode as srd

# Dictionary of FUNCTION commands and their names.
command = {
    # Scratchpad
    0x4e: 'Write scratchpad',
    0xbe: 'Read scratchpad',
    0x48: 'Copy scratchpad',
    # Thermometer
    0x44: 'Convert temperature',
    0xb4: 'Read power mode',
    0xb8: 'Recall EEPROM',
    0xf5: 'PIO access read',
    0xA5: 'PIO access write',
    0x99: 'Chain',
}

class Decoder(srd.Decoder):
    api_version = 3
    id = 'ds28ea00'
    name = 'DS28EA00'
    longname = 'Maxim DS28EA00 1-Wire digital thermometer'
    desc = '1-Wire digital thermometer with Sequence Detect and PIO.'
    license = 'gplv2+'
    inputs = ['onewire_network']
    outputs = []
    tags = ['IC', 'Sensor']
    annotations = (
        ('text', 'Human-readable text'),
    )

    def __init__(self):
        self.reset()

    def reset(self):
        self.trn_beg = 0
        self.trn_end = 0
        self.state = 'ROM'
        self.rom = 0x0000000000000000

    def start(self):
        self.out_ann = self.register(srd.OUTPUT_ANN)

    def putx(self, data):
        self.put(self.ss, self.es, self.out_ann, data)

    def decode(self, ss, es, data):
        code, val = data

        self.ss, self.es = ss, es

        # State machine.
        if code == 'RESET/PRESENCE':
            self.putx([0, ['Reset/presence: %s'
                           % ('true' if val else 'false')]])
            self.state = 'ROM'
        elif code == 'ROM':
            self.rom = val
            self.putx([0, ['ROM: 0x%016x' % (val)]])
            self.state = 'COMMAND'
        elif code == 'DATA':
            if self.state == 'COMMAND':
                if val not in command:
                    self.putx([0, ['Unrecognized command: 0x%02x' % val]])
                    return
                self.putx([0, ['Function command: 0x%02x \'%s\''
                          % (val, command[val])]])
                self.state = command[val].upper()
            elif self.state == 'READ SCRATCHPAD':
                self.putx([0, ['Scratchpad data: 0x%02x' % val]])
            elif self.state == 'CONVERT TEMPERATURE':
                self.putx([0, ['Temperature conversion status: 0x%02x' % val]])
            elif self.state in [s.upper() for s in command.values()]:
                self.putx([0, ['TODO \'%s\': 0x%02x' % (self.state, val)]])



##
## This file is part of the libsigrokdecode project.
##
## Copyright (C) 2017 Karl Palsson <karlp@etactica.com>
##
## Permission is hereby granted, free of charge, to any person obtaining a copy
## of this software and associated documentation files (the "Software"), to deal
## in the Software without restriction, including without limitation the rights
## to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
## copies of the Software, and to permit persons to whom the Software is
## furnished to do so, subject to the following conditions:
##
## The above copyright notice and this permission notice shall be included in all
## copies or substantial portions of the Software.
##
## THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
## IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
## FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
## AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
## LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
## OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
## SOFTWARE.

from collections import OrderedDict

# Generated from datasheet rev E, using tabula.
regs = OrderedDict([
    (0x1, ('AWATTHR', 'Watt-Hour Accumulation Register for Phase A. Active power is accumulated over time in this read-only register. The AWATTHR register can hold a maximum of 0.52 seconds of active energy information with full-scale analog inputs before it overflows (see the Active Energy Calculation section). Bit 0 and Bit 1 of the COMPMODE register determine how the active energy is processed from the six analog inputs.', 'R', 16, 'S', 0x0)),
    (0x2, ('BWATTHR', 'Watt-Hour Accumulation Register for Phase B.', 'R', 16, 'S', 0x0)),
    (0x3, ('CWATTHR', 'Watt-Hour Accumulation Register for Phase C.', 'R', 16, 'S', 0x0)),
    (0x4, ('AVARHR', 'VAR-Hour Accumulation Register for Phase A. Reactive power is accumulated over time in this read-only register. The AVARHR register can hold a maximum of 0.52 seconds of reactive energy information with full-scale analog inputs before it overflows (see the Reactive Energy Calculation section). Bit 0 and Bit 1 of the COMPMODE register determine how the reactive energy is processed from the six analog inputs.', 'R', 16, 'S', 0x0)),
    (0x5, ('BVARHR', 'VAR-Hour Accumulation Register for Phase B.', 'R', 16, 'S', 0x0)),
    (0x6, ('CVARHR', 'VAR-Hour Accumulation Register for Phase C.', 'R', 16, 'S', 0x0)),
    (0x7, ('AVAHR', 'VA-Hour Accumulation Register for Phase A. Apparent power is accumulated over time in this read-only register. The AVAHR register can hold a maximum of 1.15 seconds of apparent energy information with full-scale analog inputs before it overflows (see the Apparent Energy Calculation section). Bit 0 and Bit 1 of the COMPMODE register determine how the apparent energy is processed from the six analog inputs.', 'R', 16, 'S', 0x0)),
    (0x8, ('BVAHR', 'VA-Hour Accumulation Register for Phase B.', 'R', 16, 'S', 0x0)),
    (0x9, ('CVAHR', 'VA-Hour Accumulation Register for Phase C.', 'R', 16, 'S', 0x0)),
    (0xa, ('AIRMS', 'Phase A Current Channel RMS Register. The register contains the rms component of the Phase A input of the current channel. The source is selected by data bits in the mode register.', 'R', 24, 'S', 0x0)),
    (0xb, ('BIRMS', 'Phase B Current Channel RMS Register.', 'R', 24, 'S', 0x0)),
    (0xc, ('CIRMS', 'Phase C Current Channel RMS Register.', 'R', 24, 'S', 0x0)),
    (0xd, ('AVRMS', 'Phase A Voltage Channel RMS Register.', 'R', 24, 'S', 0x0)),
    (0xe, ('BVRMS', 'Phase B Voltage Channel RMS Register.', 'R', 24, 'S', 0x0)),
    (0xf, ('CVRMS', 'Phase C Voltage Channel RMS Register.', 'R', 24, 'S', 0x0)),
    (0x10, ('FREQ', 'Frequency of the Line Input Estimated by the Zero-Crossing Processing. It can also display the period of the line input. Bit 7 of the LCYCMODE register determines if the reading is frequency or period. Default is frequency. Data Bit 0 and Bit 1 of the MMODE register determine the voltage channel used for the frequency or period calculation.', 'R', 12, 'U', 0x0)),
    (0x11, ('TEMP', 'Temperature Register. This register contains the result of the latest temperature conversion. Refer to the Temperature Measurement section for details on how to interpret the content of this register.', 'R', 8, 'S', 0x0)),
    (0x12, ('WFORM', 'Waveform Register. This register contains the digitized waveform of one of the six analog inputs or the digitized power waveform. The source is selected by Data Bit 0 to Bit 4 in the WAVMODE register.', 'R', 24, 'S', 0x0)),
    (0x13, ('OPMODE', 'Operational Mode Register. This register defines the general configuration of the ADE7758 (see Table 18).', 'R/W', 8, 'U', 0x4)),
    (0x14, ('MMODE', 'Measurement Mode Register. This register defines the channel used for period and peak detection measurements (see Table 19).', 'R/W', 8, 'U', 0xfc)),
    (0x15, ('WAVMODE', 'Waveform Mode Register. This register defines the channel and sampling frequency used in the waveform sampling mode (see Table 20).', 'R/W', 8, 'U', 0x0)),
    (0x16, ('COMPMODE', 'Computation Mode Register. This register configures the formula applied for the energy and line active energy measurements (see Table 22).', 'R/W', 8, 'U', 0x1c)),
    (0x17, ('LCYCMODE', 'Line Cycle Mode Register. This register configures the line cycle accumulation mode for WATT-HR', 'R/W', 8, 'U', 0x78)),
    (0x18, ('Mask', 'IRQ Mask Register. It determines if an interrupt event generates an active-low output at the IRQ pin (see the Interrupts section).', 'R/W', 24, 'U', 0x0)),
    (0x19, ('Status', 'IRQ Status Register. This register contains information regarding the source of the ADE7758 interrupts (see the Interrupts section).', 'R', 24, 'U', 0x0)),
    (0x1a, ('RSTATUS', 'IRQ Reset Status Register. Same as the STATUS register, except that its contents are reset to 0 (all flags cleared) after a read operation.', 'R', 24, 'U', 0x0)),
    (0x1b, ('ZXTOUT', 'Zero-Cross Timeout Register. If no zero crossing is detected within the time period specified by this register', 'R/W', 16, 'U', 0xffff)),
    (0x1c, ('LINECYC', 'Line Cycle Register. The content of this register sets the number of half-line cycles that the active', 'R/W', 16, 'U', 0xffff)),
    (0x1d, ('SAGCYC', 'SAG Line Cycle Register. This register specifies the number of consecutive half-line cycles where voltage channel input may fall below a threshold level. This register is common to the three line voltage SAG detection. The detection threshold is specified by the SAGLVL register (see the Line Voltage SAG Detection section).', 'R/W', 8, 'U', 0xff)),
    (0x1e, ('SAGLVL', 'SAG Voltage Level. This register specifies the detection threshold for the SAG event. This register is common to all three phases’ line voltage SAG detections. See the description of the SAGCYC register for details.', 'R/W', 8, 'U', 0x0)),
    (0x1f, ('VPINTLVL', 'Voltage Peak Level Interrupt Threshold Register. This register sets the level of the voltage peak detection. Bit 5 to Bit 7 of the MMODE register determine which phases are to be monitored. If the selected voltage phase exceeds this level', 'R/W', 8, 'U', 0xff)),
    (0x20, ('IPINTLVL', 'Current Peak Level Interrupt Threshold Register. This register sets the level of the current peak detection. Bit 5 to Bit 7 of the MMODE register determine which phases are to be monitored. If the selected current phase exceeds this level', 'R/W', 8, 'U', 0xff)),
    (0x21, ('VPEAK', 'Voltage Peak Register. This register contains the value of the peak voltage waveform that has occurred within a fixed number of half-line cycles. The number of half-line cycles is set by the LINECYC register.', 'R', 8, 'U', 0x0)),
    (0x22, ('IPEAK', 'Current Peak Register. This register holds the value of the peak current waveform that has occurred within a fixed number of half-line cycles. The number of half-line cycles is set by the LINECYC register.', 'R', 8, 'U', 0x0)),
    (0x23, ('Gain', 'PGA Gain Register. This register is used to adjust the gain selection for the PGA in the current and voltage channels (see the Analog Inputs section).', 'R/W', 8, 'U', 0x0)),
    (0x24, ('AVRMSGAIN', 'Phase A VRMS Gain Register. The range of the voltage rms calculation can be adjusted by writing to this register. It has an adjustment range of ±50% with a resolution of 0.0244%/LSB.', 'R/W', 12, 'S', 0x0)),
    (0x25, ('BVRMSGAIN', 'Phase B VRMS Gain Register.', 'R/W', 12, 'S', 0x0)),
    (0x26, ('CVRMSGAIN', 'Phase C VRMS Gain Register.', 'R/W', 12, 'S', 0x0)),
    (0x27, ('AIGAIN', 'Phase A Current Gain Register. This register is not recommended to be used and it should be kept at 0', 'R/W', 12, 'S', 0x0)),
    (0x28, ('BIGAIN', 'Phase B Current Gain Register. This register is not recommended to be used and it should be kept at 0', 'R/W', 12, 'S', 0x0)),
    (0x29, ('CIGAIN', 'Phase C Current Gain Register. This register is not recommended to be used and it should be kept at 0', 'R/W', 12, 'S', 0x0)),
    (0x2a, ('AWG', 'Phase A Watt Gain Register. The range of the watt calculation can be adjusted by writing to this register. It has an adjustment range of ±50% with a resolution of 0.0244%/LSB.', 'R/W', 12, 'S', 0x0)),
    (0x2b, ('BWG', 'Phase B Watt Gain Register.', 'R/W', 12, 'S', 0x0)),
    (0x2c, ('CWG', 'Phase C Watt Gain Register.', 'R/W', 12, 'S', 0x0)),
    (0x2d, ('AVARG', 'Phase A VAR Gain Register. The range of the VAR calculation can be adjusted by writing to this register. It has an adjustment range of ±50% with a resolution of 0.0244%/LSB.', 'R/W', 12, 'S', 0x0)),
    (0x2e, ('BVARG', 'Phase B VAR Gain Register.', 'R/W', 12, 'S', 0x0)),
    (0x2f, ('CVARG', 'Phase C VAR Gain Register.', 'R/W', 12, 'S', 0x0)),
    (0x30, ('AVAG', 'Phase A VA Gain Register. The range of the VA calculation can be adjusted by writing to this register. It has an adjustment range of ±50% with a resolution of 0.0244%/LSB.', 'R/W', 12, 'S', 0x0)),
    (0x31, ('BVAG', 'Phase B VA Gain Register.', 'R/W', 12, 'S', 0x0)),
    (0x32, ('CVAG', 'Phase C VA Gain Register.', 'R/W', 12, 'S', 0x0)),
    (0x33, ('AVRMSOS', 'Phase A Voltage RMS Offset Correction Register.', 'R/W', 12, 'S', 0x0)),
    (0x34, ('BVRMSOS', 'Phase B Voltage RMS Offset Correction Register.', 'R/W', 12, 'S', 0x0)),
    (0x35, ('CVRMSOS', 'Phase C Voltage RMS Offset Correction Register.', 'R/W', 12, 'S', 0x0)),
    (0x36, ('AIRMSOS', 'Phase A Current RMS Offset Correction Register.', 'R/W', 12, 'S', 0x0)),
    (0x37, ('BIRMSOS', 'Phase B Current RMS Offset Correction Register.', 'R/W', 12, 'S', 0x0)),
    (0x38, ('CIRMSOS', 'Phase C Current RMS Offset Correction Register.', 'R/W', 12, 'S', 0x0)),
    (0x39, ('AWATTOS', 'Phase A Watt Offset Calibration Register.', 'R/W', 12, 'S', 0x0)),
    (0x3a, ('BWATTOS', 'Phase B Watt Offset Calibration Register.', 'R/W', 12, 'S', 0x0)),
    (0x3b, ('CWATTOS', 'Phase C Watt Offset Calibration Register.', 'R/W', 12, 'S', 0x0)),
    (0x3c, ('AVAROS', 'Phase A VAR Offset Calibration Register.', 'R/W', 12, 'S', 0x0)),
    (0x3d, ('BVAROS', 'Phase B VAR Offset Calibration Register.', 'R/W', 12, 'S', 0x0)),
    (0x3e, ('CVAROS', 'Phase C VAR Offset Calibration Register.', 'R/W', 12, 'S', 0x0)),
    (0x3f, ('APHCAL', 'Phase A Phase Calibration Register. The phase relationship between the current and voltage channel can be adjusted by writing to this signed 7-bit register (see the Phase Compensation section).', 'R/W', 7, 'S', 0x0)),
    (0x40, ('BPHCAL', 'Phase B Phase Calibration Register.', 'R/W', 7, 'S', 0x0)),
    (0x41, ('CPHCAL', 'Phase C Phase Calibration Register.', 'R/W', 7, 'S', 0x0)),
    (0x42, ('WDIV', 'Active Energy Register Divider.', 'R/W', 8, 'U', 0x0)),
    (0x43, ('VARDIV', 'Reactive Energy Register Divider.', 'R/W', 8, 'U', 0x0)),
    (0x44, ('VADIV', 'Apparent Energy Register Divider.', 'R/W', 8, 'U', 0x0)),
    (0x45, ('APCFNUM', 'Active Power CF Scaling Numerator Register. The content of this register is used in the numerator of the APCF output scaling calculation. Bits [15:13] indicate reverse polarity active power measurement for Phase A', 'R/W', 16, 'U', 0x0)),
    (0x46, ('APCFDEN', 'Active Power CF Scaling Denominator Register. The content of this register is used in the denominator of the APCF output scaling.', 'R/W', 12, 'U', 0x3f)),
    (0x47, ('VARCFNUM', 'Reactive Power CF Scaling Numerator Register. The content of this register is used in the numerator of the VARCF output scaling. Bits [15:13] indicate reverse polarity reactive power measurement for Phase A', 'R/W', 16, 'U', 0x0)),
    (0x48, ('VARCFDEN', 'Reactive Power CF Scaling Denominator Register. The content of this register is used in the denominator of the VARCF output scaling.', 'R/W', 12, 'U', 0x3f)),
    (0x7e, ('CHKSUM', 'Checksum Register. The content of this register represents the sum of all the ones in the last register read from the SPI port.', 'R', 8, 'U', None)),
    (0x7f, ('Version', 'Version of the Die.', 'R', 8, 'U', None)),
])



##
## This file is part of the libsigrokdecode project.
##
## Copyright (C) 2016 Rudolf Reuter <reuterru@arcor.de>
##
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, see <http://www.gnu.org/licenses/>.
##

import sigrokdecode as srd

class Decoder(srd.Decoder):
    api_version = 3
    id = 'gpib'
    name = 'GPIB'
    longname = 'General Purpose Interface Bus'
    desc = 'IEEE-488 GPIB / HPIB protocol.'
    license = 'gplv2+'
    inputs = ['logic']
    outputs = ['gpib']
    channels = (
        {'id': 'dio1' , 'name': 'DIO1', 'desc': 'Data I/O bit 1'},
        {'id': 'dio2' , 'name': 'DIO2', 'desc': 'Data I/O bit 2'},
        {'id': 'dio3' , 'name': 'DIO3', 'desc': 'Data I/O bit 3'},
        {'id': 'dio4' , 'name': 'DIO4', 'desc': 'Data I/O bit 4'},
        {'id': 'dio5' , 'name': 'DIO5', 'desc': 'Data I/O bit 5'},
        {'id': 'dio6' , 'name': 'DIO6', 'desc': 'Data I/O bit 6'},
        {'id': 'dio7' , 'name': 'DIO7', 'desc': 'Data I/O bit 7'},
        {'id': 'dio8' , 'name': 'DIO8', 'desc': 'Data I/O bit 8'},
        {'id': 'eoi', 'name': 'EOI', 'desc': 'End or identify'},
        {'id': 'dav', 'name': 'DAV', 'desc': 'Data valid'},
        {'id': 'nrfd', 'name': 'NRFD', 'desc': 'Not ready for data'},
        {'id': 'ndac', 'name': 'NDAC', 'desc': 'Not data accepted'},
        {'id': 'ifc', 'name': 'IFC', 'desc': 'Interface clear'},
        {'id': 'srq', 'name': 'SRQ', 'desc': 'Service request'},
        {'id': 'atn', 'name': 'ATN', 'desc': 'Attention'},
        {'id': 'ren', 'name': 'REN', 'desc': 'Remote enable'},
    )
    options = (
        {'id': 'sample_total', 'desc': 'Total number of samples', 'default': 0},
    )
    annotations = (
        ('items', 'Items'),
        ('gpib', 'DAT/CMD'),
        ('eoi', 'EOI'),
    )
    annotation_rows = (
        ('bytes', 'Bytes', (0,)),
        ('gpib', 'DAT/CMD', (1,)),
        ('eoi', 'EOI', (2,)),
    )

    def __init__(self):
        self.reset()

    def reset(self):
        self.items = []
        self.itemcount = 0
        self.saved_item = None
        self.saved_ATN = False
        self.saved_EOI = False
        self.samplenum = 0
        self.ss_item = self.es_item = None
        self.first = True

    def start(self):
        self.out_ann = self.register(srd.OUTPUT_ANN)

    def putb(self, data):
        self.put(self.ss_item, self.es_item, self.out_ann, data)

    def handle_bits(self, datapins):
        dbyte = 0x20
        dATN = False
        item2 = False
        dEOI = False
        item3 = False
        # If this is the first item in a word, save its sample number.
        if self.itemcount == 0:
            self.ss_word = self.samplenum

        # Get the bits for this item.
        item = 0
        for i in range(8):
            item |= datapins[i] << i

        item = item ^ 0xff # Invert data byte.
        self.items.append(item)
        self.itemcount += 1

        if datapins[14] == 0:
            item2 = True
        if datapins[8] == 0:
            item3 = True

        if self.first:
            # Save the start sample and item for later (no output yet).
            self.ss_item = self.samplenum
            self.first = False
            self.saved_item = item
            self.saved_ATN = item2
            self.saved_EOI = item3
        else:
            # Output the saved item.
            dbyte = self.saved_item
            dATN = self.saved_ATN
            dEOI = self.saved_EOI
            self.es_item = self.samplenum
            self.putb([0, ['%02X' % self.saved_item]])

            # Encode item byte to GPIB convention.
            self.strgpib = ' '
            if dATN: # ATN, decode commands.
                if dbyte == 0x01: self.strgpib = 'GTL'
                if dbyte == 0x04: self.strgpib = 'SDC'
                if dbyte == 0x05: self.strgpib = 'PPC'
                if dbyte == 0x08: self.strgpib = 'GET'
                if dbyte == 0x09: self.strgpib = 'TCT'
                if dbyte == 0x11: self.strgpib = 'LLO'
                if dbyte == 0x14: self.strgpib = 'DCL'
                if dbyte == 0x15: self.strgpib = 'PPU'
                if dbyte == 0x18: self.strgpib = 'SPE'
                if dbyte == 0x19: self.strgpib = 'SPD'
                if dbyte == 0x3f: self.strgpib = 'UNL'
                if dbyte == 0x5f: self.strgpib = 'UNT'
                if dbyte > 0x1f and dbyte < 0x3f: # Address Listener.
                    self.strgpib = 'L' + chr(dbyte + 0x10)
                if dbyte > 0x3f and dbyte < 0x5f: # Address Talker
                    self.strgpib = 'T' + chr(dbyte - 0x10)
            else:
                if dbyte > 0x1f and dbyte < 0x7f:
                    self.strgpib = chr(dbyte)
                if dbyte == 0x0a:
                    self.strgpib = 'LF'
                if dbyte == 0x0d:
                    self.strgpib = 'CR'

            self.putb([1, [self.strgpib]])
            self.strEOI = ' '
            if dEOI:
                self.strEOI = 'EOI'
            self.putb([2, [self.strEOI]])

            self.ss_item = self.samplenum
            self.saved_item = item
            self.saved_ATN = item2
            self.saved_EOI = item3

        if self.itemcount < 16:
            return

        self.itemcount, self.items = 0, []

    def decode(self):

        # Inspect samples at falling edge of DAV. But make sure to also
        # start inspection when the capture happens to start with low
        # DAV level. Optionally enforce processing when a user specified
        # sample number was reached.
        waitcond = [{9: 'l'}]
        lsn = self.options['sample_total']
        if lsn:
            waitcond.append({'skip': lsn})
        while True:
            if lsn:
                waitcond[1]['skip'] = lsn - self.samplenum - 1
            pins = self.wait(waitcond)
            self.handle_bits(pins)
            waitcond[0][9] = 'f'



##
## This file is part of the libsigrokdecode project.
##
## Copyright (C) 2019 Benedikt Otto <benedikt_o@web.de>
##
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, see <http://www.gnu.org/licenses/>.
##

import sigrokdecode as srd

class SamplerateError(Exception):
    pass

class Decoder(srd.Decoder):
    api_version = 3
    id = 'ir_rc6'
    name = 'IR RC-6'
    longname = 'IR RC-6'
    desc = 'RC-6 infrared remote control protocol.'
    license = 'gplv2+'
    inputs = ['logic']
    outputs = []
    tags = ['IR']
    channels = (
        {'id': 'ir', 'name': 'IR', 'desc': 'IR data line'},
    )
    options = (
        {'id': 'polarity', 'desc': 'Polarity', 'default': 'auto',
            'values': ('auto', 'active-low', 'active-high')},
    )
    annotations = (
        ('bit', 'Bit'),
        ('sync', 'Sync'),
        ('startbit', 'Startbit'),
        ('field', 'Field'),
        ('togglebit', 'Togglebit'),
        ('address', 'Address'),
        ('command', 'Command'),
    )
    annotation_rows = (
        ('bits', 'Bits', (0,)),
        ('fields', 'Fields', (1, 2, 3, 4, 5, 6)),
    )

    def __init__(self):
        self.reset()

    def reset(self):
        self.samplerate = None
        self.edges, self.deltas, self.bits = [], [], []
        self.state = 'IDLE'
        self.mode = 0

    def start(self):
        self.out_ann = self.register(srd.OUTPUT_ANN)

    def metadata(self, key, value):
        if key == srd.SRD_CONF_SAMPLERATE:
            self.samplerate = value
            # One bit: 0.889ms (one half low, one half high).
            self.halfbit = int((self.samplerate * 0.000889) / 2.0)

    def putb(self, bit, data):
        self.put(bit[0], bit[1], self.out_ann, data)

    def putbits(self, bit1, bit2, data):
        self.put(bit1[0], bit2[1], self.out_ann, data)

    def putx(self, ss, es, data):
        self.put(ss, es, self.out_ann, data)

    def handle_bit(self):
        if len(self.bits) != 6:
            return
        if self.bits[0][2] == 8 and self.bits[0][3] == 1:
            self.putb(self.bits[0], [1, ['Synchronisation', 'Sync']])
        else:
            return
        if self.bits[1][3] == 1:
            self.putb(self.bits[1], [2, ['Startbit', 'Start']])
        else:
            return
        self.mode = sum([self.bits[2 + i][3] << (2 - i) for i in range(3)])
        self.putbits(self.bits[2], self.bits[4], [3, ['Field: %d' % self.mode]])
        self.putb(self.bits[5], [4, ['Toggle: %d' % self.bits[5][3]]])

    def handle_package(self):
        # Sync and start bits have to be 1.
        if self.bits[0][3] == 0 or self.bits[1][3] == 0:
            return
        if len(self.bits) <= 6:
            return

        if self.mode == 0 and len(self.bits) == 22: # Mode 0 standard
            value = sum([self.bits[6 + i][3] << (7 - i) for i in range(8)])
            self.putbits(self.bits[6], self.bits[13], [5, ['Address: %0.2X' % value]])

            value = sum([self.bits[14 + i][3] << (7 - i) for i in range(8)])
            self.putbits(self.bits[14], self.bits[21], [6, ['Data: %0.2X' % value]])

            self.bits = []

        if self.mode == 6 and len(self.bits) >= 15: # Mode 6
            if self.bits[6][3] == 0: # Short addr, Mode 6A
                value = sum([self.bits[6 + i][3] << (7 - i) for i in range(8)])
                self.putbits(self.bits[6], self.bits[13], [5, ['Address: %0.2X' % value]])

                num_data_bits = len(self.bits) - 14
                value = sum([self.bits[14 + i][3] << (num_data_bits - 1 - i) for i in range(num_data_bits)])
                self.putbits(self.bits[14], self.bits[-1], [6, ['Data: %X' % value]])

                self.bits = []

            elif len(self.bits) >= 23: # Long addr, Mode 6B
                value = sum([self.bits[6 + i][3] << (15 - i) for i in range(16)])
                self.putbits(self.bits[6], self.bits[21], [5, ['Address: %0.2X' % value]])

                num_data_bits = len(self.bits) - 22
                value = sum([self.bits[22 + i][3] << (num_data_bits - 1 - i) for i in range(num_data_bits)])
                self.putbits(self.bits[22], self.bits[-1], [6, ['Data: %X' % value]])

                self.bits = []

    def decode(self):
        if not self.samplerate:
            raise SamplerateError('Cannot decode without samplerate.')
        value = 0
        num_edges = -1
        self.invert = False

        while True:
            conditions = [{0: 'e'}]
            if self.state == 'DATA':
                conditions.append({'skip': self.halfbit * 6})
            (self.ir,) = self.wait(conditions)

            if len(conditions) == 2:
                if self.matched[1]:
                    self.state = 'IDLE'

            self.edges.append(self.samplenum)
            if len(self.edges) < 2:
                continue

            delta = (self.edges[-1] - self.edges[-2]) / self.halfbit
            delta = int(delta + 0.5)
            self.deltas.append(delta)

            if len(self.deltas) < 2:
                continue

            if self.deltas[-2:] == [6, 2]:
                self.state = 'SYNC'
                num_edges = 0
                self.bits = []

                if self.options['polarity'] == 'auto':
                    value = 1
                else:
                    value = self.ir if self.options['polarity'] == 'active-high' else 1 - self.ir

                self.bits.append((self.edges[-3], self.edges[-1], 8, value))
                self.invert = self.ir == 0
                self.putb(self.bits[-1], [0, ['%d' % value]]) # Add bit.

            if (num_edges % 2) == 0: # Only count every second edge.
                if self.deltas[-2] in [1, 2, 3] and self.deltas[-1] in [1, 2, 3, 6]:
                    self.state = 'DATA'
                    if self.deltas[-2] != self.deltas[-1]:
                        # Insert border between 2 bits.
                        self.edges.insert(-1, self.edges[-2] + self.deltas[-2] * self.halfbit)
                        total = self.deltas[-1]
                        self.deltas[-1] = self.deltas[-2]
                        self.deltas.append(total - self.deltas[-1])

                        self.bits.append((self.edges[-4], self.edges[-2], self.deltas[-2] * 2, value))

                        num_edges += 1
                    else:
                        self.bits.append((self.edges[-3], self.edges[-1], self.deltas[-1] * 2, value))

                    self.putb(self.bits[-1], [0, ['%d' % value]]) # Add bit.

            if len(self.bits) > 0:
                self.handle_bit()
                if self.state == 'IDLE':
                    self.handle_package()

            if self.options['polarity'] == 'auto':
                value = self.ir if self.invert else 1 - self.ir
            else:
                value = self.ir if self.options['polarity'] == 'active-low' else 1 - self.ir

            num_edges += 1



##
## This file is part of the libsigrokdecode project.
##
## Copyright (C) 2016 Sean Burford <sburford@google.com>
##
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, see <http://www.gnu.org/licenses/>.
##

import sigrokdecode as srd

class SamplerateError(Exception):
    pass

class Decoder(srd.Decoder):
    api_version = 3
    id = 'wiegand'
    name = 'Wiegand'
    longname = 'Wiegand interface'
    desc = 'Wiegand interface for electronic entry systems.'
    license = 'gplv2+'
    inputs = ['logic']
    outputs = []
    tags = ['Embedded/industrial', 'RFID']
    channels = (
        {'id': 'd0', 'name': 'D0', 'desc': 'Data 0 line'},
        {'id': 'd1', 'name': 'D1', 'desc': 'Data 1 line'},
    )
    options = (
        {'id': 'active', 'desc': 'Data lines active level',
         'default': 'low', 'values': ('low', 'high')},
        {'id': 'bitwidth_ms', 'desc': 'Single bit width in milliseconds',
         'default': 4, 'values': (1, 2, 4, 8, 16, 32)},
    )
    annotations = (
        ('bits', 'Bits'),
        ('state', 'State'),
    )
    annotation_rows = (
        ('bits', 'Binary value', (0,)),
        ('state', 'Stream state', (1,)),
    )

    def __init__(self):
        self.reset()

    def reset(self):
        self.samplerate = None
        self._samples_per_bit = 10

        self._d0_prev = None
        self._d1_prev = None

        self._state = None
        self.ss_state = None

        self.ss_bit = None
        self.es_bit = None
        self._bit = None
        self._bits = []

    def start(self):
        'Register output types and verify user supplied decoder values.'
        self.out_ann = self.register(srd.OUTPUT_ANN)
        self._active = 1 if self.options['active'] == 'high' else 0
        self._inactive = 1 - self._active

    def metadata(self, key, value):
        'Receive decoder metadata about the data stream.'
        if key == srd.SRD_CONF_SAMPLERATE:
            self.samplerate = value
            if self.samplerate:
                ms_per_sample = 1000 * (1.0 / self.samplerate)
                ms_per_bit = float(self.options['bitwidth_ms'])
                self._samples_per_bit = int(max(1, int(ms_per_bit / ms_per_sample)))

    def _update_state(self, state, bit=None):
        'Update state and bit values when they change.'
        if self._bit is not None:
            self._bits.append(self._bit)
            self.put(self.ss_bit, self.samplenum, self.out_ann,
                     [0, [str(self._bit)]])
        self._bit = bit
        self.ss_bit = self.samplenum
        if bit is not None:
            # Set a timeout so that the final bit ends.
            self.es_bit = self.samplenum + self._samples_per_bit
        else:
            self.es_bit = None

        if state != self._state:
            ann = None
            if self._state == 'data':
                accum_bits = ''.join(str(x) for x in self._bits)
                ann = [1, ['%d bits %s' % (len(self._bits), accum_bits),
                           '%d bits' % len(self._bits)]]
            elif self._state == 'invalid':
                ann = [1, [self._state]]
            if ann:
                self.put(self.ss_state, self.samplenum, self.out_ann, ann)
            self.ss_state = self.samplenum
            self._state = state
            self._bits = []

    def decode(self):
        if not self.samplerate:
            raise SamplerateError('Cannot decode without samplerate.')
        while True:
            # TODO: Come up with more appropriate self.wait() conditions.
            (d0, d1) = self.wait()

            if d0 == self._d0_prev and d1 == self._d1_prev:
                if self.es_bit and self.samplenum >= self.es_bit:
                    if (d0, d1) == (self._inactive, self._inactive):
                        self._update_state('idle')
                    else:
                        self._update_state('invalid')
                continue

            if self._state in (None, 'idle', 'data'):
                if (d0, d1) == (self._active, self._inactive):
                    self._update_state('data', 0)
                elif (d0, d1) == (self._inactive, self._active):
                    self._update_state('data', 1)
                elif (d0, d1) == (self._active, self._active):
                    self._update_state('invalid')
            elif self._state == 'invalid':
                # Wait until we see an idle state before leaving invalid.
                # This prevents inverted lines from being misread.
                if (d0, d1) == (self._inactive, self._inactive):
                    self._update_state('idle')

            self._d0_prev, self._d1_prev = d0, d1

    def report(self):
        return '%s: %s D0 %d D1 %d (active on %d), %d samples per bit' % (
            self.name, self._state, self._d0_prev, self._d1_prev,
            self._active, self._samples_per_bit)



##
## This file is part of the libsigrokdecode project.
##
## Copyright (C) 2014 Uwe Hermann <uwe@hermann-uwe.de>
##
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, see <http://www.gnu.org/licenses/>.
##

# Systems/addresses (0..31). Items that are not listed are reserved/unknown.
system = {
    0: ['TV receiver 1', 'TV1'],
    1: ['TV receiver 2', 'TV2'],
    2: ['Teletext', 'Txt'],
    3: ['Extension to TV1 and TV2', 'Ext TV1/TV2'],
    4: ['LaserVision player', 'LV'],
    5: ['Video cassette recorder 1', 'VCR1'],
    6: ['Video cassette recorder 2', 'VCR2'],
    7: ['Experimental', 'Exp'],
    8: ['Satellite TV receiver 1', 'Sat1'],
    9: ['Extension to VCR1 and VCR2', 'Ext VCR1/VCR2'],
    10: ['Satellite TV receiver 2', 'Sat2'],
    12: ['Compact disc video player', 'CD-Video'],
    13: ['Camcorder', 'Cam'],
    14: ['Photo on compact disc player', 'CD-Photo'],
    16: ['Audio preamplifier 1', 'Preamp1'],
    17: ['Radio tuner', 'Tuner'],
    18: ['Analog cassette recoder 1', 'Rec1'],
    19: ['Audio preamplifier 2', 'Preamp2'],
    20: ['Compact disc player', 'CD'],
    21: ['Audio stack or record player', 'Combi'],
    22: ['Audio satellite', 'Sat'],
    23: ['Analog cassette recoder 2', 'Rec2'],
    26: ['Compact disc recorder', 'CD-R'],
    29: ['Lighting 1', 'Light1'],
    30: ['Lighting 2', 'Light2'],
    31: ['Telephone', 'Phone'],
}

digits = {
    0: ['0', '0'],
    1: ['1', '1'],
    2: ['2', '2'],
    3: ['3', '3'],
    4: ['4', '4'],
    5: ['5', '5'],
    6: ['6', '6'],
    7: ['7', '7'],
    8: ['8', '8'],
    9: ['9', '9'],
}

# Commands (0..63 for RC-5, and 0..127 for Extended RC-5).
# Items that are not listed are reserved/unknown.
command = {
    'TV': dict(list(digits.items()) + list({
        10: ['-/--', '-/--'],
        11: ['Channel/program', 'Ch/P'],
        12: ['Standby', 'StBy'],
        13: ['Mute', 'M'],
        14: ['Personal preferences', 'PP'],
        15: ['Display', 'Disp'],
        16: ['Volume up', 'Vol+'],
        17: ['Volume down', 'Vol-'],
        18: ['Brightness up', 'Br+'],
        19: ['Brightness down', 'Br-'],
        20: ['Saturation up', 'S+'],
        21: ['Saturation down', 'S-'],
        32: ['Program up', 'P+'],
        33: ['Program down', 'P-'],
    }.items())),
    'VCR': dict(list(digits.items()) + list({
        10: ['-/--', '-/--'],
        12: ['Standby', 'StBy'],
        32: ['Program up', 'P+'],
        33: ['Program down', 'P-'],
        50: ['Fast rewind', 'FRW'],
        52: ['Fast forward', 'FFW'],
        53: ['Play', 'Pl'],
        54: ['Stop', 'St'],
        55: ['Recording', 'Rec'],
    }.items())),
}



##
## This file is part of the libsigrokdecode project.
##
## Copyright (C) 2017 Kevin Redon <kingkevin@cuvoodoo.info>
##
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, see <http://www.gnu.org/licenses/>.
##

import sigrokdecode as srd

class SamplerateError(Exception):
    pass

# Timing values in us for the signal at regular and overdrive speed.
timing = {
    'RSTL': {
        'min': {
            False: 480.0,
            True: 48.0,
        },
        'max': {
            False: 960.0,
            True: 80.0,
        },
    },
    'RSTH': {
        'min': {
            False: 480.0,
            True: 48.0,
        },
    },
    'PDH': {
        'min': {
            False: 15.0,
            True: 2.0,
        },
        'max': {
            False: 60.0,
            True: 6.0,
        },
    },
    'PDL': {
        'min': {
            False: 60.0,
            True: 8.0,
        },
        'max': {
            False: 240.0,
            True: 24.0,
        },
    },
    'SLOT': {
        'min': {
            False: 60.0,
            True: 6.0,
        },
        'max': {
            False: 120.0,
            True: 16.0,
        },
    },
    'REC': {
        'min': {
            False: 1.0,
            True: 1.0,
        },
    },
    'LOWR': {
        'min': {
            False: 1.0,
            True: 1.0,
        },
        'max': {
            False: 15.0,
            True: 2.0,
        },
    },
}

class Decoder(srd.Decoder):
    api_version = 3
    id = 'onewire_link'
    name = '1-Wire link layer'
    longname = '1-Wire serial communication bus (link layer)'
    desc = 'Bidirectional, half-duplex, asynchronous serial bus.'
    license = 'gplv2+'
    inputs = ['logic']
    outputs = ['onewire_link']
    tags = ['Embedded/industrial']
    channels = (
        {'id': 'owr', 'name': 'OWR', 'desc': '1-Wire signal line'},
    )
    options = (
        {'id': 'overdrive', 'desc': 'Start in overdrive speed',
            'default': 'no', 'values': ('yes', 'no')},
    )
    annotations = (
        ('bit', 'Bit'),
        ('warnings', 'Warnings'),
        ('reset', 'Reset'),
        ('presence', 'Presence'),
        ('overdrive', 'Overdrive speed notifications'),
    )
    annotation_rows = (
        ('bits', 'Bits', (0, 2, 3)),
        ('info', 'Info', (4,)),
        ('warnings', 'Warnings', (1,)),
    )

    def __init__(self):
        self.reset()

    def reset(self):
        self.samplerate = None
        self.state = 'INITIAL'
        self.present = 0
        self.bit = 0
        self.bit_count = -1
        self.command = 0
        self.overdrive = False
        self.fall = 0
        self.rise = 0

    def start(self):
        self.out_python = self.register(srd.OUTPUT_PYTHON)
        self.out_ann = self.register(srd.OUTPUT_ANN)
        self.overdrive = (self.options['overdrive'] == 'yes')
        self.fall = 0
        self.rise = 0
        self.bit_count = -1

    def putm(self, data):
        self.put(0, 0, self.out_ann, data)

    def putpfs(self, data):
        self.put(self.fall, self.samplenum, self.out_python, data)

    def putfs(self, data):
        self.put(self.fall, self.samplenum, self.out_ann, data)

    def putfr(self, data):
        self.put(self.fall, self.rise, self.out_ann, data)

    def putprs(self, data):
        self.put(self.rise, self.samplenum, self.out_python, data)

    def putrs(self, data):
        self.put(self.rise, self.samplenum, self.out_ann, data)

    def checks(self):
        # Check if samplerate is appropriate.
        if self.options['overdrive'] == 'yes':
            if self.samplerate < 2000000:
                self.putm([1, ['Sampling rate is too low. Must be above ' +
                               '2MHz for proper overdrive mode decoding.']])
            elif self.samplerate < 5000000:
                self.putm([1, ['Sampling rate is suggested to be above 5MHz ' +
                               'for proper overdrive mode decoding.']])
        else:
            if self.samplerate < 400000:
                self.putm([1, ['Sampling rate is too low. Must be above ' +
                               '400kHz for proper normal mode decoding.']])
            elif self.samplerate < 1000000:
                self.putm([1, ['Sampling rate is suggested to be above ' +
                               '1MHz for proper normal mode decoding.']])

    def metadata(self, key, value):
        if key != srd.SRD_CONF_SAMPLERATE:
            return
        self.samplerate = value

    def wait_falling_timeout(self, start, t):
        # Wait until either a falling edge is seen, and/or the specified
        # number of samples have been skipped (i.e. time has passed).
        cnt = int((t[self.overdrive] / 1000000.0) * self.samplerate)
        samples_to_skip = (start + cnt) - self.samplenum
        samples_to_skip = samples_to_skip if (samples_to_skip > 0) else 0
        return self.wait([{0: 'f'}, {'skip': samples_to_skip}])

    def decode(self):
        if not self.samplerate:
            raise SamplerateError('Cannot decode without samplerate.')
        self.checks()
        while True:
            # State machine.
            if self.state == 'INITIAL': # Unknown initial state.
                # Wait until we reach the idle high state.
                self.wait({0: 'h'})
                self.rise = self.samplenum
                self.state = 'IDLE'
            elif self.state == 'IDLE': # Idle high state.
                # Wait for falling edge.
                self.wait({0: 'f'})
                self.fall = self.samplenum
                # Get time since last rising edge.
                time = ((self.fall - self.rise) / self.samplerate) * 1000000.0
                if self.rise > 0 and \
                    time < timing['REC']['min'][self.overdrive]:
                    self.putfr([1, ['Recovery time not long enough'
                        'Recovery too short',
                        'REC < ' + str(timing['REC']['min'][self.overdrive])]])
                # A reset pulse or slot can start on a falling edge.
                self.state = 'LOW'
                # TODO: Check minimum recovery time.
            elif self.state == 'LOW': # Reset pulse or slot.
                # Wait for rising edge.
                self.wait({0: 'r'})
                self.rise = self.samplenum
                # Detect reset or slot base on timing.
                time = ((self.rise - self.fall) / self.samplerate) * 1000000.0
                if time >= timing['RSTL']['min'][False]: # Normal reset pulse.
                    if time > timing['RSTL']['max'][False]:
                        self.putfr([1, ['Too long reset pulse might mask interrupt ' +
                            'signalling by other devices',
                            'Reset pulse too long',
                            'RST > ' + str(timing['RSTL']['max'][False])]])
                    # Regular reset pulse clears overdrive speed.
                    if self.overdrive:
                        self.putfr([4, ['Exiting overdrive mode', 'Overdrive off']])
                    self.overdrive = False
                    self.putfr([2, ['Reset', 'Rst', 'R']])
                    self.state = 'PRESENCE DETECT HIGH'
                elif self.overdrive == True and \
                    time >= timing['RSTL']['min'][self.overdrive] and \
                    time < timing['RSTL']['max'][self.overdrive]:
                    # Overdrive reset pulse.
                    self.putfr([2, ['Reset', 'Rst', 'R']])
                    self.state = 'PRESENCE DETECT HIGH'
                elif time < timing['SLOT']['max'][self.overdrive]:
                    # Read/write time slot.
                    if time < timing['LOWR']['min'][self.overdrive]:
                        self.putfr([1, ['Low signal not long enough',
                            'Low too short',
                            'LOW < ' + str(timing['LOWR']['min'][self.overdrive])]])
                    if time < timing['LOWR']['max'][self.overdrive]:
                        self.bit = 1 # Short pulse is a 1 bit.
                    else:
                        self.bit = 0 # Long pulse is a 0 bit.
                    # Wait for end of slot.
                    self.state = 'SLOT'
                else:
                    # Timing outside of known states.
                    self.putfr([1, ['Erroneous signal', 'Error', 'Err', 'E']])
                    self.state = 'IDLE'
            elif self.state == 'PRESENCE DETECT HIGH': # Wait for slave presence signal.
                # Wait for a falling edge and/or presence detect signal.
                self.wait_falling_timeout(self.rise, timing['PDH']['max'])

                # Calculate time since rising edge.
                time = ((self.samplenum - self.rise) / self.samplerate) * 1000000.0

                if self.matched[0] and not self.matched[1]:
                    # Presence detected.
                    if time < timing['PDH']['min'][self.overdrive]:
                        self.putrs([1, ['Presence detect signal is too early',
                            'Presence detect too early',
                            'PDH < ' + str(timing['PDH']['min'][self.overdrive])]])
                    self.fall = self.samplenum
                    self.state = 'PRESENCE DETECT LOW'
                else: # No presence detected.
                    self.putrs([3, ['Presence: false', 'Presence', 'Pres', 'P']])
                    self.putprs(['RESET/PRESENCE', False])
                    self.state = 'IDLE'
            elif self.state == 'PRESENCE DETECT LOW': # Slave presence signalled.
                # Wait for end of presence signal (on rising edge).
                self.wait({0: 'r'})
                # Calculate time since start of presence signal.
                time = ((self.samplenum - self.fall) / self.samplerate) * 1000000.0
                if time < timing['PDL']['min'][self.overdrive]:
                    self.putfs([1, ['Presence detect signal is too short',
                        'Presence detect too short',
                        'PDL < ' + str(timing['PDL']['min'][self.overdrive])]])
                elif time > timing['PDL']['max'][self.overdrive]:
                    self.putfs([1, ['Presence detect signal is too long',
                        'Presence detect too long',
                        'PDL > ' + str(timing['PDL']['max'][self.overdrive])]])
                if time > timing['RSTH']['min'][self.overdrive]:
                    self.rise = self.samplenum
                # Wait for end of presence detect.
                self.state = 'PRESENCE DETECT'

            # End states (for additional checks).
            if self.state == 'SLOT': # Wait for end of time slot.
                # Wait for a falling edge and/or end of timeslot.
                self.wait_falling_timeout(self.fall, timing['SLOT']['min'])

                if self.matched[0] and not self.matched[1]:
                    # Low detected before end of slot.
                    self.putfs([1, ['Time slot not long enough',
                        'Slot too short',
                        'SLOT < ' + str(timing['SLOT']['min'][self.overdrive])]])
                    # Don't output invalid bit.
                    self.fall = self.samplenum
                    self.state = 'LOW'
                else: # End of time slot.
                    # Output bit.
                    self.putfs([0, ['Bit: %d' % self.bit, '%d' % self.bit]])
                    self.putpfs(['BIT', self.bit])
                    # Save command bits.
                    if self.bit_count >= 0:
                        self.command += (self.bit << self.bit_count)
                        self.bit_count += 1
                    # Check for overdrive ROM command.
                    if self.bit_count >= 8:
                        if self.command == 0x3c or self.command == 0x69:
                            self.overdrive = True
                            self.put(self.samplenum, self.samplenum,
                                self.out_ann,
                                [4, ['Entering overdrive mode', 'Overdrive on']])
                        self.bit_count = -1
                    self.state = 'IDLE'

            if self.state == 'PRESENCE DETECT':
                # Wait for a falling edge and/or end of presence detect.
                self.wait_falling_timeout(self.rise, timing['RSTH']['min'])

                if self.matched[0] and not self.matched[1]:
                    # Low detected before end of presence detect.
                    self.putfs([1, ['Presence detect not long enough',
                        'Presence detect too short',
                        'RTSH < ' + str(timing['RSTH']['min'][self.overdrive])]])
                    # Inform about presence detected.
                    self.putrs([3, ['Slave presence detected', 'Slave present',
                        'Present', 'P']])
                    self.putprs(['RESET/PRESENCE', True])
                    self.fall = self.samplenum
                    self.state = 'LOW'
                else: # End of time slot.
                    # Inform about presence detected.
                    self.putrs([3, ['Presence: true', 'Presence', 'Pres', 'P']])
                    self.putprs(['RESET/PRESENCE', True])
                    self.rise = self.samplenum
                    # Start counting the first 8 bits to get the ROM command.
                    self.bit_count = 0
                    self.command = 0
                    self.state = 'IDLE'



##
## This file is part of the libsigrokdecode project.
##
## Copyright (C) 2011 Gareth McMullin <gareth@blacksphere.co.nz>
## Copyright (C) 2012-2014 Uwe Hermann <uwe@hermann-uwe.de>
##
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, see <http://www.gnu.org/licenses/>.
##

import sigrokdecode as srd
from collections import namedtuple

Data = namedtuple('Data', ['ss', 'es', 'val'])

'''
OUTPUT_PYTHON format:

Packet:
[<ptype>, <data1>, <data2>]

<ptype>:
 - 'DATA': <data1> contains the MOSI data, <data2> contains the MISO data.
   The data is _usually_ 8 bits (but can also be fewer or more bits).
   Both data items are Python numbers (not strings), or None if the respective
   channel was not supplied.
 - 'BITS': <data1>/<data2> contain a list of bit values in this MOSI/MISO data
   item, and for each of those also their respective start-/endsample numbers.
 - 'CS-CHANGE': <data1> is the old CS# pin value, <data2> is the new value.
   Both data items are Python numbers (0/1), not strings. At the beginning of
   the decoding a packet is generated with <data1> = None and <data2> being the
   initial state of the CS# pin or None if the chip select pin is not supplied.
 - 'TRANSFER': <data1>/<data2> contain a list of Data() namedtuples for each
   byte transferred during this block of CS# asserted time. Each Data() has
   fields ss, es, and val.

Examples:
 ['CS-CHANGE', None, 1]
 ['CS-CHANGE', 1, 0]
 ['DATA', 0xff, 0x3a]
 ['BITS', [[1, 80, 82], [1, 83, 84], [1, 85, 86], [1, 87, 88],
           [1, 89, 90], [1, 91, 92], [1, 93, 94], [1, 95, 96]],
          [[0, 80, 82], [1, 83, 84], [0, 85, 86], [1, 87, 88],
           [1, 89, 90], [1, 91, 92], [0, 93, 94], [0, 95, 96]]]
 ['DATA', 0x65, 0x00]
 ['DATA', 0xa8, None]
 ['DATA', None, 0x55]
 ['CS-CHANGE', 0, 1]
 ['TRANSFER', [Data(ss=80, es=96, val=0xff), ...],
              [Data(ss=80, es=96, val=0x3a), ...]]
'''

# Key: (CPOL, CPHA). Value: SPI mode.
# Clock polarity (CPOL) = 0/1: Clock is low/high when inactive.
# Clock phase (CPHA) = 0/1: Data is valid on the leading/trailing clock edge.
spi_mode = {
    (0, 0): 0, # Mode 0
    (0, 1): 1, # Mode 1
    (1, 0): 2, # Mode 2
    (1, 1): 3, # Mode 3
}

class ChannelError(Exception):
    pass

class Decoder(srd.Decoder):
    api_version = 3
    id = 'spi'
    name = 'SPI'
    longname = 'Serial Peripheral Interface'
    desc = 'Full-duplex, synchronous, serial bus.'
    license = 'gplv2+'
    inputs = ['logic']
    outputs = ['spi']
    tags = ['Embedded/industrial']
    channels = (
        {'id': 'clk', 'name': 'CLK', 'desc': 'Clock'},
    )
    optional_channels = (
        {'id': 'miso', 'name': 'MISO', 'desc': 'Master in, slave out'},
        {'id': 'mosi', 'name': 'MOSI', 'desc': 'Master out, slave in'},
        {'id': 'cs', 'name': 'CS#', 'desc': 'Chip-select'},
    )
    options = (
        {'id': 'cs_polarity', 'desc': 'CS# polarity', 'default': 'active-low',
            'values': ('active-low', 'active-high')},
        {'id': 'cpol', 'desc': 'Clock polarity', 'default': 0,
            'values': (0, 1)},
        {'id': 'cpha', 'desc': 'Clock phase', 'default': 0,
            'values': (0, 1)},
        {'id': 'bitorder', 'desc': 'Bit order',
            'default': 'msb-first', 'values': ('msb-first', 'lsb-first')},
        {'id': 'wordsize', 'desc': 'Word size', 'default': 8},
    )
    annotations = (
        ('miso-data', 'MISO data'),
        ('mosi-data', 'MOSI data'),
        ('miso-bits', 'MISO bits'),
        ('mosi-bits', 'MOSI bits'),
        ('warnings', 'Human-readable warnings'),
        ('miso-transfer', 'MISO transfer'),
        ('mosi-transfer', 'MOSI transfer'),
    )
    annotation_rows = (
        ('miso-bits', 'MISO bits', (2,)),
        ('miso-data', 'MISO data', (0,)),
        ('miso-transfer', 'MISO transfer', (5,)),
        ('mosi-bits', 'MOSI bits', (3,)),
        ('mosi-data', 'MOSI data', (1,)),
        ('mosi-transfer', 'MOSI transfer', (6,)),
        ('other', 'Other', (4,)),
    )
    binary = (
        ('miso', 'MISO'),
        ('mosi', 'MOSI'),
    )

    def __init__(self):
        self.reset()

    def reset(self):
        self.samplerate = None
        self.bitcount = 0
        self.misodata = self.mosidata = 0
        self.misobits = []
        self.mosibits = []
        self.misobytes = []
        self.mosibytes = []
        self.ss_block = -1
        self.ss_transfer = -1
        self.cs_was_deasserted = False
        self.have_cs = self.have_miso = self.have_mosi = None

    def start(self):
        self.out_python = self.register(srd.OUTPUT_PYTHON)
        self.out_ann = self.register(srd.OUTPUT_ANN)
        self.out_binary = self.register(srd.OUTPUT_BINARY)
        self.out_bitrate = self.register(srd.OUTPUT_META,
                meta=(int, 'Bitrate', 'Bitrate during transfers'))
        self.bw = (self.options['wordsize'] + 7) // 8

    def metadata(self, key, value):
       if key == srd.SRD_CONF_SAMPLERATE:
            self.samplerate = value

    def putw(self, data):
        self.put(self.ss_block, self.samplenum, self.out_ann, data)

    def putdata(self):
        # Pass MISO and MOSI bits and then data to the next PD up the stack.
        so = self.misodata if self.have_miso else None
        si = self.mosidata if self.have_mosi else None
        so_bits = self.misobits if self.have_miso else None
        si_bits = self.mosibits if self.have_mosi else None

        if self.have_miso:
            ss, es = self.misobits[-1][1], self.misobits[0][2]
            bdata = so.to_bytes(self.bw, byteorder='big')
            self.put(ss, es, self.out_binary, [0, bdata])
        if self.have_mosi:
            ss, es = self.mosibits[-1][1], self.mosibits[0][2]
            bdata = si.to_bytes(self.bw, byteorder='big')
            self.put(ss, es, self.out_binary, [1, bdata])

        self.put(ss, es, self.out_python, ['BITS', si_bits, so_bits])
        self.put(ss, es, self.out_python, ['DATA', si, so])

        if self.have_miso:
            self.misobytes.append(Data(ss=ss, es=es, val=so))
        if self.have_mosi:
            self.mosibytes.append(Data(ss=ss, es=es, val=si))

        # Bit annotations.
        if self.have_miso:
            for bit in self.misobits:
                self.put(bit[1], bit[2], self.out_ann, [2, ['%d' % bit[0]]])
        if self.have_mosi:
            for bit in self.mosibits:
                self.put(bit[1], bit[2], self.out_ann, [3, ['%d' % bit[0]]])

        # Dataword annotations.
        if self.have_miso:
            self.put(ss, es, self.out_ann, [0, ['%02X' % self.misodata]])
        if self.have_mosi:
            self.put(ss, es, self.out_ann, [1, ['%02X' % self.mosidata]])

    def reset_decoder_state(self):
        self.misodata = 0 if self.have_miso else None
        self.mosidata = 0 if self.have_mosi else None
        self.misobits = [] if self.have_miso else None
        self.mosibits = [] if self.have_mosi else None
        self.bitcount = 0

    def cs_asserted(self, cs):
        active_low = (self.options['cs_polarity'] == 'active-low')
        return (cs == 0) if active_low else (cs == 1)

    def handle_bit(self, miso, mosi, clk, cs):
        # If this is the first bit of a dataword, save its sample number.
        if self.bitcount == 0:
            self.ss_block = self.samplenum
            self.cs_was_deasserted = \
                not self.cs_asserted(cs) if self.have_cs else False

        ws = self.options['wordsize']
        bo = self.options['bitorder']

        # Receive MISO bit into our shift register.
        if self.have_miso:
            if bo == 'msb-first':
                self.misodata |= miso << (ws - 1 - self.bitcount)
            else:
                self.misodata |= miso << self.bitcount

        # Receive MOSI bit into our shift register.
        if self.have_mosi:
            if bo == 'msb-first':
                self.mosidata |= mosi << (ws - 1 - self.bitcount)
            else:
                self.mosidata |= mosi << self.bitcount

        # Guesstimate the endsample for this bit (can be overridden below).
        es = self.samplenum
        if self.bitcount > 0:
            if self.have_miso:
                es += self.samplenum - self.misobits[0][1]
            elif self.have_mosi:
                es += self.samplenum - self.mosibits[0][1]

        if self.have_miso:
            self.misobits.insert(0, [miso, self.samplenum, es])
        if self.have_mosi:
            self.mosibits.insert(0, [mosi, self.samplenum, es])

        if self.bitcount > 0 and self.have_miso:
            self.misobits[1][2] = self.samplenum
        if self.bitcount > 0 and self.have_mosi:
            self.mosibits[1][2] = self.samplenum

        self.bitcount += 1

        # Continue to receive if not enough bits were received, yet.
        if self.bitcount != ws:
            return

        self.putdata()

        # Meta bitrate.
        if self.samplerate:
            elapsed = 1 / float(self.samplerate)
            elapsed *= (self.samplenum - self.ss_block + 1)
            bitrate = int(1 / elapsed * ws)
            self.put(self.ss_block, self.samplenum, self.out_bitrate, bitrate)

        if self.have_cs and self.cs_was_deasserted:
            self.putw([4, ['CS# was deasserted during this data word!']])

        self.reset_decoder_state()

    def find_clk_edge(self, miso, mosi, clk, cs, first):
        if self.have_cs and (first or self.matched[self.have_cs]):
            # Send all CS# pin value changes.
            oldcs = None if first else 1 - cs
            self.put(self.samplenum, self.samplenum, self.out_python,
                     ['CS-CHANGE', oldcs, cs])

            if self.cs_asserted(cs):
                self.ss_transfer = self.samplenum
                self.misobytes = []
                self.mosibytes = []
            elif self.ss_transfer != -1:
                if self.have_miso:
                    self.put(self.ss_transfer, self.samplenum, self.out_ann,
                        [5, [' '.join(format(x.val, '02X') for x in self.misobytes)]])
                if self.have_mosi:
                    self.put(self.ss_transfer, self.samplenum, self.out_ann,
                        [6, [' '.join(format(x.val, '02X') for x in self.mosibytes)]])
                self.put(self.ss_transfer, self.samplenum, self.out_python,
                    ['TRANSFER', self.mosibytes, self.misobytes])

            # Reset decoder state when CS# changes (and the CS# pin is used).
            self.reset_decoder_state()

        # We only care about samples if CS# is asserted.
        if self.have_cs and not self.cs_asserted(cs):
            return

        # Ignore sample if the clock pin hasn't changed.
        if first or not self.matched[0]:
            return

        # Sample data on rising/falling clock edge (depends on mode).
        mode = spi_mode[self.options['cpol'], self.options['cpha']]
        if mode == 0 and clk == 0:   # Sample on rising clock edge
            return
        elif mode == 1 and clk == 1: # Sample on falling clock edge
            return
        elif mode == 2 and clk == 1: # Sample on falling clock edge
            return
        elif mode == 3 and clk == 0: # Sample on rising clock edge
            return

        # Found the correct clock edge, now get the SPI bit(s).
        self.handle_bit(miso, mosi, clk, cs)

    def decode(self):
        # The CLK input is mandatory. Other signals are (individually)
        # optional. Yet either MISO or MOSI (or both) must be provided.
        # Tell stacked decoders when we don't have a CS# signal.
        if not self.has_channel(0):
            raise ChannelError('Either MISO or MOSI (or both) pins required.')
        self.have_miso = self.has_channel(1)
        self.have_mosi = self.has_channel(2)
        if not self.have_miso and not self.have_mosi:
            raise ChannelError('Either MISO or MOSI (or both) pins required.')
        self.have_cs = self.has_channel(3)
        if not self.have_cs:
            self.put(0, 0, self.out_python, ['CS-CHANGE', None, None])

        # We want all CLK changes. We want all CS changes if CS is used.
        # Map 'have_cs' from boolean to an integer index. This simplifies
        # evaluation in other locations.
        wait_cond = [{0: 'e'}]
        if self.have_cs:
            self.have_cs = len(wait_cond)
            wait_cond.append({3: 'e'})

        # "Pixel compatibility" with the v2 implementation. Grab and
        # process the very first sample before checking for edges. The
        # previous implementation did this by seeding old values with
        # None, which led to an immediate "change" in comparison.
        (clk, miso, mosi, cs) = self.wait({})
        self.find_clk_edge(miso, mosi, clk, cs, True)

        while True:
            (clk, miso, mosi, cs) = self.wait(wait_cond)
            self.find_clk_edge(miso, mosi, clk, cs, False)
